VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "D3D8Engine"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit

' Direct Draw variables
Private dx As DirectX8                          ' The main DX object
Private SpriteVerts(3) As TLVERTEX              ' Vertex information for this sprite.
Private d3d As Direct3D8                        ' The direct 3d object
Private dev As Direct3DDevice8                  ' The main device
Private d3dx As D3DX8                           ' Helper library
Private oSurfaces As New Collection             ' holds all loaded surfaces
Private VertBuffer As Direct3DVertexBuffer8     ' Holds vertice buffer
Private VertSize As Integer                     ' The size of one vertice

' For point Sprites
Private PrtData(0 To nParticles - 1) As PARTICLE
Private PrtVertList(0 To nParticles - 1) As TLVERTEX

' View matrixes
Private matWorld As D3DMATRIX
Private matView As D3DMATRIX
Private matProj As D3DMATRIX

' Light variables
Private Lights() As D3DLIGHT8           ' Holds all the lights data
Private Ambient As Long                 ' Ambient light
Public aHwnd As Long                    ' stores the window handle of our main window

' Font variable
Private fnt As New StdFont
Private tColor As Long
Private D3DFont As D3DXFont             ' The Font
Private CurrentCustomFont As String     ' Current texture being used for Custom Fonts

' Holds the data for the FPS system
Private FPSLastCheck As Long
Private FPSCount As Long
Private FPSCurrent As Long

' Starts up the particle engine
Public Sub InitParticles()

Dim i As Integer

' Sets up each particle individually
For i = 0 To nParticles - 1
    PrtData(i).Status = True
    
    ' Randomizes the X-Position
    If Rnd < 0.5 Then
        PrtData(i).x = -(Rnd * 6)
    Else
        PrtData(i).x = (Rnd * 6)
    End If
    
    ' Randomizes the Y position
    If Rnd < 0.5 Then
        PrtData(i).y = -(Rnd * 6)
    Else
        PrtData(i).y = (Rnd * 6)
    End If
    
    ' No Depth
    PrtData(i).Z = 0
    
    ' Gives slight variations in the X and Y speed
    PrtData(i).vX = -0.05 - (Rnd * 0.25)
    PrtData(i).vY = -4 + (Rnd * 0.5)
    PrtData(i).vZ = 0
    
    ' Update the vertices data
    PrtVertList(i).x = PrtData(i).x
    PrtVertList(i).y = PrtData(i).y
    PrtVertList(i).Z = PrtData(i).Z

Next i


'Enable point sprite rendering
dev.SetRenderState D3DRS_POINTSPRITE_ENABLE, 1 '//Enable point sprite rendering

'Allow Direct3D to size the points
dev.SetRenderState D3DRS_POINTSCALE_ENABLE, 1 '//Allow Direct3D to set/alter the size of the Psprites

'Set the size
dev.SetRenderState D3DRS_POINTSIZE, 1000524822

End Sub

' Actually render the particles
Public Sub RenderParticles()
Dim i As Long

    For i = 0 To nParticles - 1
        
        If PrtData(i).Status = True Then
                ' Update the positions
                PrtData(i).x = PrtData(i).x + ((PrtData(i).vX / 500) * 10)
                PrtData(i).y = PrtData(i).y + ((PrtData(i).vY / 500) * 10)
                    
                ' Check if the particle has gone below ground level...
                If Abs(PrtData(i).y) > 6 Or Abs(PrtData(i).x) > 6 Or Abs(PrtData(i).x) > 6 Then PrtData(i).Status = False
        Else
                ' We need to recreate our particle...
                PrtData(i).Status = True
                If Rnd < 0.5 Then
                    PrtData(i).x = -(Rnd * 6)
                Else
                    PrtData(i).x = (Rnd * 6)
                End If
                
                PrtData(i).y = 6
                PrtData(i).Z = 0
                PrtData(i).vX = -0.05 - (Rnd * 0.25)
                PrtData(i).vY = -4 + (Rnd * 0.5)
                PrtData(i).vZ = 0
        End If
        
        ' Update the Verts
        PrtVertList(i).x = PrtData(i).x
        PrtVertList(i).y = PrtData(i).y
        PrtVertList(i).Z = PrtData(i).Z

    Next i
    
    ' Set the texture to the default Particle texture (Must load it externally)
    dev.SetTexture 0, oSurfaces("Particle")
    
    ' Actually draw them
    dev.DrawPrimitiveUP D3DPT_POINTLIST, nParticles, PrtVertList(0), Len(PrtVertList(0))
    
End Sub

' Get the number of lights
Public Property Get NumberOfLights() As Byte
    NumberOfLights = UBound(Lights())
End Property

' Loads an offscreen surface into memory and stores in a collection
Public Function LoadSurface(sName As String, sFilename As String, Optional ColorKeyValue As Long = COLOR_BLACK) As Boolean
Dim tmpSurface As Direct3DTexture8

    On Error GoTo ErrorHandler
    
    ' It automatically detects the size of the bitmap
    Set tmpSurface = d3dx.CreateTextureFromFileEx(dev, sFilename, D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED, D3DX_FILTER_POINT, D3DX_FILTER_POINT, ColorKeyValue, ByVal 0, ByVal 0)

    ' Add the VBSurface object to our collection
    oSurfaces.Add tmpSurface, sName
    
    ' Success
    LoadSurface = True
    
    Exit Function
    
ErrorHandler:
    LoadSurface = False
End Function

' Brightens the ambient light
Public Sub Brighten()
    
    ' If already brightest keep it there
    If Ambient >= &HD0D0D0 Then
        Ambient = &HFFFFFF
    Else
        ' Else increment it
        Ambient = Ambient + &H202020
    End If
    
    ' Set it
    dev.SetRenderState D3DRS_AMBIENT, Ambient

End Sub

' Darkens the ambient light
Public Sub Darken()
    
    ' If already pitch black exit
    If Ambient <= &H101010 Then
        Exit Sub
    End If
    
    ' Darken it
    Ambient = Ambient - &H70707
    
    ' Set it
    dev.SetRenderState D3DRS_AMBIENT, Ambient

End Sub

' Destroys a surface in the collection
Public Function KillSurface(sName As String)
Dim aSurface As Direct3DTexture8  ' The temporary surface
    
On Error GoTo ErrorHandler
    
    ' Set the temporary surface as a reference to the one will destroy
    Set aSurface = oSurfaces(sName)
            
    ' Destory the surface
    Set aSurface = Nothing
    
    ' Remove it from the collection
    oSurfaces.Remove sName
    
    ' Success
    KillSurface = True
    
    Exit Function
ErrorHandler:
    
    ' Failure
    KillSurface = False
End Function

' End the class
Public Function TerminateD3D()
Dim i As Integer
Dim count As Integer
Dim aSurface As Direct3DTexture8  ' The temporary surface

    Set D3DFont = Nothing

    count = oSurfaces.count
    i = count
    
    Do While i > 0

        Set aSurface = oSurfaces(i)

        Set aSurface = Nothing
    
        oSurfaces.Remove i
    
        i = i - 1
    
    Loop
    
    Set dev = Nothing
    Set d3d = Nothing
    Set dx = Nothing
End Function

' Draws the nonwalkable tiles among other stuff (Used for editing and debugging)
Private Sub DrawMapData(tmpMap As clsMap)
Dim x As Integer
Dim y As Integer
Dim lx As Long, ly As Long
Dim tmpTile As Integer
Dim XStart As Integer
Dim YStart As Integer
Dim lOX As Long
Dim lOY As Long
Dim CurrentX As Integer, CurrentY As Integer
Dim i As Integer
Dim tmpLight As LightPos

    ' determine the first (top left) tile to draw
    XStart = tmpMap.MapOffsetX \ TILEWIDTH
    YStart = tmpMap.MapOffsetY \ TILEWIDTH
    
    ' determine the pixel offset of the first tile
    lOX = tmpMap.MapOffsetX Mod TILEWIDTH: lOX = -lOX
    lOY = tmpMap.MapOffsetY Mod TILEHEIGHT: lOY = -lOY

    
    For y = 0 To RESY \ TILEHEIGHT
        For x = 0 To RESX \ TILEWIDTH
            
            ' Get the id for the tile
            CurrentX = XStart + x
            CurrentY = YStart + y
            
            ' Check if it is round
            If tmpMap.IsRound Then

                ' If it is check if next tile must be wrapped from the other side

                If CurrentX > tmpMap.Width - 1 Then
                    CurrentX = XStart + x - tmpMap.Width
                ElseIf CurrentX < 0 Then
                    CurrentX = tmpMap.Width + CurrentX
                End If
            
                If CurrentY > tmpMap.Height - 1 Then
                    CurrentY = YStart + y - tmpMap.Height
                ElseIf CurrentY < 0 Then
                    CurrentY = tmpMap.Height + CurrentY
                End If
            Else
                If CurrentY > tmpMap.Height - 1 Then
                    CurrentY = tmpMap.Height - 1
                End If
                
                If CurrentX > tmpMap.Width - 1 Then
                    CurrentX = tmpMap.Width - 1
                End If
                
            End If
            
            ' Get tile data
            tmpTile = tmpMap.GetTileData(CurrentX, CurrentY)
                        
            ' Find drawing offset
            lx = x * TILEWIDTH + lOX
            ly = y * TILEHEIGHT + lOY
                

            ' If nonwalkable then draw blocked pic
            If tmpTile = Blocked Then
                    
                SetUpGeometry lx / TILEWIDTH, ly / TILEHEIGHT, 0, 0, 32, 32

                ' Blt the tile
                DrawSurface3D ("SpecialData")

            ElseIf tmpTile <> 0 Then ' else label data
                
                Select Case tmpMap.GetTileData(CurrentX, CurrentY)
                
                    Case Warp
                        PlotText "W", lx + TILEWIDTH / 4, ly + TILEHEIGHT / 4, COLOR_BLACK
                    
                    Case Treasure
                        PlotText "T", lx + TILEWIDTH / 4, ly + TILEHEIGHT / 4, COLOR_BLACK
                    
                    Case TreasureChest
                        PlotText "TC", lx + TILEWIDTH / 4, ly + TILEHEIGHT / 4, COLOR_BLACK
                    
                    Case HiddenTreasure
                        PlotText "HT", lx + TILEWIDTH / 4, ly + TILEHEIGHT / 4, COLOR_BLACK
                    
                    Case SignString
                        PlotText "S", lx + TILEWIDTH / 4, ly + TILEHEIGHT / 4, COLOR_BLACK
                    
                    Case Moveable
                        PlotText "M", lx + TILEWIDTH / 4, ly + TILEHEIGHT / 4, COLOR_BLACK
                End Select
            
            End If
                        
        Next x
    Next y
    
        
    i = tmpMap.NumberOfLights
    
    Dim tmpX As Integer
    Dim tmpY As Integer
    

    ' Label the Lights
    For y = 0 To i - 1
    
        tmpX = tmpMap.GetLightTileX(y) - XStart
        tmpY = tmpMap.GetLightTileY(y) - YStart
    
        If tmpMap.IsRound Then
            If tmpX > tmpMap.Width - RESX / TILEWIDTH Then
                tmpX = XStart + tmpX
            End If
            
            If tmpY > tmpMap.Height - RESY / TILEHEIGHT Then
                tmpY = YStart + tmpY
            End If
                
        End If
    
        PlotText "L", tmpX * 32 + lOX, tmpY * 32 + lOY, COLOR_RED
    Next y
    
End Sub

' Draws the maps
Public Sub DrawAll(tmpMap As clsMap, Optional drawCharacters As Boolean = True)
Dim MapWidth As Long, MapHeight As Long
Dim lOffsetX As Long
Dim lOffsetY As Long
Dim ActChar As Integer


    lOffsetX = tmpMap.MapOffsetX
    lOffsetY = tmpMap.MapOffsetY
    MapWidth = tmpMap.Width * TILEWIDTH
    MapHeight = tmpMap.Height * TILEHEIGHT
    ActChar = tmpMap.GetActiveCharacter
    
    ' Only draw if visible
    If tmpMap.BottomVisible Then
        DrawMapBottom tmpMap
    End If
    
    ' Only draw if visible
    If tmpMap.MiddleVisible Then
        DrawMapMiddle tmpMap
    End If
    
    ' Only draw if visible
    If drawCharacters Then
        Dim i As Integer
        Dim NumChars As Integer

        NumChars = tmpMap.NumberOfCharacters
        
        i = 0
        Do While i < NumChars
            
            ' Only draw if the character is set up and being used
            If tmpMap.IsCharacterSetUp(i) Then
                If i = ActChar Then
                    DrawMainChar tmpMap.Character_Data(i)
                Else
                    DrawChar tmpMap.Character_Data(i), lOffsetX, lOffsetY, MapWidth, MapHeight
                End If
            End If
                
            i = i + 1
        Loop
        
        NumChars = tmpMap.NumberOfNPCs
        
        i = 0
        
        Do While i < NumChars
            
            ' Only draw if the character is set up and being used
            DrawNPC tmpMap.NPC_Data(i), lOffsetX, lOffsetY, MapWidth, MapHeight
                
            i = i + 1
        Loop
        
    End If
    
    ' Only draw if visible
    If tmpMap.TopVisible Then
        DrawMapTop tmpMap
    End If
    
    ' Only draw if visible
    If tmpMap.SpecialVisible Then
        DrawMapData tmpMap
    End If
    
End Sub

' Initializes the engine
Public Sub InitD3D(Optional bWindowed As Boolean = False)

Dim DispMode As D3DDISPLAYMODE
Dim D3DWindow As D3DPRESENT_PARAMETERS

    ' Initialize the devices
    Set dx = New DirectX8
    Set d3d = dx.Direct3DCreate()
    Set d3dx = New D3DX8

    ' We only want one backbuffer
    D3DWindow.BackBufferCount = 1

    ' Sets the target for drawing
    D3DWindow.hDeviceWindow = aHwnd

    If bWindowed Then

            ' Get the current display mode
            d3d.GetAdapterDisplayMode D3DADAPTER_DEFAULT, DispMode

            'For windowed mode, we just discard any information instead of flipping it.
            D3DWindow.SwapEffect = D3DSWAPEFFECT_DISCARD
                        
            'Set windowed mode to true.
            D3DWindow.Windowed = 1
            
            ' Set up the format
            D3DWindow.BackBufferFormat = DispMode.Format
        Else

            'Call the sub to find the first suitable fullscreen format
            FindMode RESX, RESY, D3DWindow.BackBufferFormat
        
            'We need the backbuffer to flip with the front for fullscreen. This
            'flag enables this.
            D3DWindow.SwapEffect = D3DSWAPEFFECT_FLIP
                                    
            'Set the width and height
            D3DWindow.BackBufferWidth = RESX
            D3DWindow.BackBufferHeight = RESY
            
            
        End If

    ' Sets up the main device, we use software vertex processing, since it has little
    ' effect on performance( might need to change for slow CPU machines to hardware)
    Set dev = d3d.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, aHwnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, D3DWindow)

    ' Set up the flexible vertex format (Const in modConst.bas)
    dev.SetVertexShader FVF
    
    ' Enable alpha blending for transparency
    dev.SetRenderState D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA
    dev.SetRenderState D3DRS_ALPHABLENDENABLE, True
    
    'Create the vertices for the Sprite
    Ambient = &HFFFFFF
    dev.SetRenderState D3DRS_AMBIENT, Ambient 'The ambient value is a hex-RRGGBB code

    ' Setup the matrixes
    Dim matWorld As D3DMATRIX
    
    ' Get and set the World Matrix
    D3DXMatrixIdentity matWorld
    dev.SetTransform D3DTS_WORLD, matWorld

    ' View matrix
    D3DXMatrixLookAtLH matView, MakeVector(0, 0, 9), MakeVector(0, 0, 0), MakeVector(0, 1, 0)
    dev.SetTransform D3DTS_VIEW, matView

    ' Projection matrix
    D3DXMatrixPerspectiveFovLH matProj, 1, 1, 0, 5
    dev.SetTransform D3DTS_PROJECTION, matProj
    
    ' Make the normals of all the vertexes
    SpriteVerts(0).nz = 1
    SpriteVerts(1).nz = 1
    SpriteVerts(2).nz = 1
    SpriteVerts(3).nz = 1
    
    ' Need at least one light
    ReDim Lights(1) As D3DLIGHT8
    
    Dim tmpLong As Long
    VertSize = Len(SpriteVerts(0))
    
    tmpLong = RESX / TILEWIDTH + 2
    
    Set VertBuffer = dev.CreateVertexBuffer(VertSize * 4 * tmpLong, 0, FVF, D3DPOOL_DEFAULT)

    CurrentCustomFont = "MainFont"

End Sub

Private Sub ResetBuffer(Optional tmpVal As Long = -1)

    Dim tmpLong As Long
    
    VertSize = Len(SpriteVerts(0))
    
    If tmpVal = -1 Then
        tmpLong = RESX / TILEWIDTH + 2
    Else
        tmpLong = tmpVal
    End If
    
    Set VertBuffer = Nothing
    Set VertBuffer = dev.CreateVertexBuffer(VertSize * 4 * tmpLong, 0, FVF, D3DPOOL_DEFAULT)


End Sub

' Need this to start drawing
Public Sub StartRender()
        
    'The app is ready for rendering.
    With dev
                
        'Clear the back buffer
        .Clear 0, ByVal 0, D3DCLEAR_TARGET, 0, 1#, 0 '//Clear the screen black
            
        'Begin the 3d scene
        .BeginScene

    End With

End Sub

' Need to end the drawing befor next frame
Public Sub EndRender()
            
    With dev
        'End the scene
        .EndScene
            
        'Draw the graphics to the front buffer.
        .Present ByVal 0&, ByVal 0&, 0, ByVal 0&
            
    End With

End Sub

' Draws the surface with the current vertex information
Private Sub DrawSurface3D(SurfaceName As String)
    
    With dev
        
        'Set the Sprite texture on the device
        .SetTexture 0, oSurfaces(SurfaceName)
                                                  
        'Draw the 2 polygons that make up the Sprite
        .DrawPrimitiveUP D3DPT_TRIANGLESTRIP, 2, SpriteVerts(0), Len(SpriteVerts(0))
  
    End With

End Sub

' Sets up the source and destination coordinates
Private Sub SetUpGeometry(ByVal StartX As Double, ByVal StartY As Double, ByVal tOffsetX As Double, ByVal tOffsety As Double, ByVal SizeX As Double, ByVal SizeY As Double)

        ' .X is the X coordinate
        ' .Y is the Y coordinate
        '
        '   See figure for coord sys.  Notice 0,0 is in center
        '
        '   (4,4)
        '   |------------|
        '   |            |
        '   |    (0,0)   |
        '   |            |
        '   |____________|
        '                (-4,-4)
        '
        ' .tu is the source X, can only be 0 to 1.
        '      (Ex. if want halfway through a bitmap with 256 pixels wide, .tu = .5
        ' .tv is the source Y, can only be 0 to 1.
        SpriteVerts(0).x = 5 - (StartX + 1) * changeX
        SpriteVerts(0).y = 5 - StartY * changeY
        SpriteVerts(0).tu = (tOffsetX + TILEWIDTH) / SizeX
        SpriteVerts(0).tv = tOffsety / SizeY
             
        SpriteVerts(1).x = 5 - (StartX + 1) * changeX
        SpriteVerts(1).y = 5 - (StartY + 1) * changeY
        SpriteVerts(1).tu = (tOffsetX + TILEWIDTH) / SizeX
        SpriteVerts(1).tv = (tOffsety + TILEHEIGHT) / SizeY
                
        SpriteVerts(2).x = 5 - StartX * changeX
        SpriteVerts(2).y = 5 - StartY * changeY
        SpriteVerts(2).tu = tOffsetX / SizeX
        SpriteVerts(2).tv = tOffsety / SizeY
                
        SpriteVerts(3).x = 5 - StartX * changeX
        SpriteVerts(3).y = 5 - (StartY + 1) * changeY
        SpriteVerts(3).tu = tOffsetX / SizeX
        SpriteVerts(3).tv = (tOffsety + TILEHEIGHT) / SizeY
    
End Sub

Private Sub ResetGeometry()
        SpriteVerts(0).x = 5
        SpriteVerts(0).y = 5
        SpriteVerts(0).tu = 0
        SpriteVerts(0).tv = 0
             
        SpriteVerts(1).x = 5
        SpriteVerts(1).y = 5
        SpriteVerts(1).tu = 0
        SpriteVerts(1).tv = 0
                
        SpriteVerts(2).x = 5
        SpriteVerts(2).y = 5
        SpriteVerts(2).tu = 0
        SpriteVerts(2).tv = 0
                
        SpriteVerts(3).x = 5
        SpriteVerts(3).y = 5
        SpriteVerts(3).tu = 0
        SpriteVerts(3).tv = 0
End Sub



' Finds the ID of the display mode wanted
Private Function FindMode(ByVal w As Long, ByVal h As Long, fmt As Long) As Long
    
    Dim i  As Long, lCount As Long
    Dim d3ddm As D3DDISPLAYMODE
    Dim bFoundMode As Boolean
    
    i = 0
    
    'Get the number of adapter modes this adapter supports.
    lCount = d3d.GetAdapterModeCount(D3DADAPTER_DEFAULT) - 1
    
    'If we encounter an error, return an error code and exit the function.
    If Err.Number Then
        FindMode = D3DERR_INVALIDCALL
        Exit Function
    End If
    
    'Next, loop through all the display modes until we find one
    'that matches the parameters passed in.
    For i = 0 To lCount
        
        Call d3d.EnumAdapterModes(D3DADAPTER_DEFAULT, i, d3ddm)
        
        'Again, catch any unexpected errors.
        If Err.Number Then
            FindMode = Err.Number
            Exit Function
        End If
        
        'Check to see if this mode matches what is being sought.
        If d3ddm.Width = w And d3ddm.Height = h Then
            
            'Now see if this mode is either a 32bpp or 16bpp mode
            If d3ddm.Format = D3DFMT_R8G8B8 Or _
                d3ddm.Format = D3DFMT_R5G6B5 Then
                
                'We've found a suitable display. Set the flag
                'to reflect this, and exit. No need to look further.
                bFoundMode = True
                
                'Set the fmt to the format that was found.
                fmt = d3ddm.Format
                
                Exit For
            End If
        End If
    Next
        
    If bFoundMode Then
        
        'Everything checked out OK
        Exit Function
        
    Else
        
        'Return an error
        FindMode = D3DERR_INVALIDDEVICE
        
    End If
    
End Function


' Returns Frames Per Second (Uses win-API)
Public Function FPS() As Long

    ' get the FPS
    FPSCount = FPSCount + 1
    
    If FPSCount = 10 Then
            FPSCurrent = (10000 / (GetTickCount() - FPSLastCheck))
            FPSLastCheck = GetTickCount()
            FPSCount = 0
    End If
    
    FPS = FPSCurrent
    
End Function

' Plots text at a specific Pixel Coord
Public Sub PlotText(sText As String, Optional StartX As Long = 0, Optional StartY As Long = 0, Optional lColor As Long = 0, Optional CustomFont = False)
Dim tmpRect As RECT
    
    ' Set up the destination rectangle
    With tmpRect
        .Left = StartX
        .Right = RESX
        .Top = StartY
        .Bottom = StartY + TILEHEIGHT
    End With
    
    If Not CustomFont Then
    
        ' Turn off AlphaBlend (Need to do this for some reason)
        dev.SetRenderState D3DRS_ALPHABLENDENABLE, False
    
        ' Draw it
        d3dx.DrawText D3DFont, lColor, sText, tmpRect, DT_TOP

        ' Enable AlphaBlend
        dev.SetRenderState D3DRS_ALPHABLENDENABLE, True
    Else
        PlotCustomFont sText, StartX, StartY, fnt.Size, fnt.Size * 0.75
    End If

End Sub

' Set up the font
Public Function SetUpFont(Optional sName As String = "Arial", Optional fSize As Integer = 12, Optional bBold As Boolean = False)

' Temporary variables used to initialize it
Dim FontDesc As IFont
 
    fnt.Name = sName
    fnt.Size = fSize
    fnt.Bold = bBold
    
    Set FontDesc = fnt
    Set D3DFont = d3dx.CreateFont(dev, FontDesc.hFont)

End Function

' Draws the sprite on the backbuffer
Public Sub DrawMainChar(Char As Character)
    
    SetUpGeometry Char.PosX / TILEWIDTH, Char.PosY / TILEHEIGHT, Char.iOffsetX, Char.iOffsetY, Char.TilesWide * TILEWIDTH, Char.TilesHigh * TILEHEIGHT
        
    DrawSurface3D Char.SurfaceName
         
End Sub

' Draws the sprite on the backbuffer
Public Sub DrawChar(Char As Character, MapOffsetX As Long, MapOffsetY As Long, MapWidth As Long, MapHeight As Long)
Dim CharX As Long, CharY As Long

CharX = Char.WorldX
CharY = Char.WorldY
    
    ' Convert Position to current view position
    If MapOffsetX > MapWidth - RESX Then
        If CharX > MapOffsetX Then
            CharX = CharX - MapOffsetX
        Else
            CharX = CharX + (MapWidth - MapOffsetX)
        End If
    Else
        CharX = CharX - MapOffsetX
    End If

    If MapOffsetY > MapHeight - RESY Then
        If CharY > MapOffsetY Then
            CharY = CharY - MapOffsetY
        Else
            CharY = CharY + (MapHeight - MapOffsetY)
        End If
    Else
        CharY = CharY - MapOffsetY
    End If

    ' Not visible then don't draw
    If CharX < 0 Or CharX > RESX Then
        Exit Sub
    ElseIf CharY < 0 Or CharY > RESY Then
        Exit Sub
    End If
    
    SetUpGeometry CharX / TILEWIDTH, CharY / TILEHEIGHT, Char.iOffsetX, Char.iOffsetY, Char.TilesWide * TILEWIDTH, Char.TilesHigh * TILEHEIGHT
    DrawSurface3D Char.SurfaceName
    
           
End Sub

Public Sub DrawNPC(Char As NPC_Character, MapOffsetX As Long, MapOffsetY As Long, MapWidth As Long, MapHeight As Long)
Dim CharX As Long, CharY As Long

CharX = Char.WorldX
CharY = Char.WorldY
    
    SpriteVerts(0).Z = 0.001
    SpriteVerts(1).Z = 0.001
    SpriteVerts(2).Z = 0.001
    SpriteVerts(3).Z = 0.001
    
    ' Convert to current view location
    If MapOffsetX > MapWidth - RESX Then
        If CharX > MapOffsetX Then
            CharX = CharX - MapOffsetX
        Else
            CharX = CharX + (MapWidth - MapOffsetX)
        End If
    Else
        CharX = CharX - MapOffsetX
    End If

    If MapOffsetY > MapHeight - RESY Then
        If CharY > MapOffsetY Then
            CharY = CharY - MapOffsetY
        Else
            CharY = CharY + (MapHeight - MapOffsetY)
        End If
    Else
        CharY = CharY - MapOffsetY
    End If

    ' If not visible exit
    If CharX > RESX Or CharX < 0 Then Exit Sub
    If CharY > RESY Or CharY < 0 Then Exit Sub

    SetUpGeometry CharX / TILEWIDTH, CharY / TILEHEIGHT, Char.iOffsetX, Char.iOffsetY, Char.TilesWide * TILEWIDTH, Char.TilesHigh * TILEHEIGHT
    DrawSurface3D Char.SurfaceName

    SpriteVerts(0).Z = 0
    SpriteVerts(1).Z = 0
    SpriteVerts(2).Z = 0
    SpriteVerts(3).Z = 0
         
End Sub

' Draws the sprite on the backbuffer
Public Sub DrawSprite(sName As String, PosX As Long, PosY As Long, lOffsetX As Long, lOffsetY As Long, lSizeX As Long, lSizeY As Long)
    
    If PosX > RESX Or PosX < 0 Then Exit Sub
    If PosY > RESY Or PosY < 0 Then Exit Sub
    
    SetUpGeometry PosX / TILEWIDTH, PosY / TILEHEIGHT, lOffsetX, lOffsetY, lSizeX, lSizeY
    DrawSurface3D sName
    
End Sub


' Draws the text box (Must load text surface) (Everything is in percent)
Public Sub DrawBox(Optional pHeight As Long = 20, Optional pWidth As Long = 100, Optional XPos As Long = 0, Optional YPos As Long = 0)
    Dim iHeight As Long, iWidth As Long
    Dim i As Integer, j As Integer
    
    iHeight = (pHeight / 100) * (RESY / TILEHEIGHT) - 1
    iWidth = (pWidth / 100) * (RESX / TILEWIDTH)
    
    ResetBuffer iWidth + 1
    
    ' Set up and draw the upper left corner
    SetUpGeometry 0, YPos, 0, 0, 96, 96
    D3DVertexBuffer8SetData VertBuffer, 0, VertSize * 4, 0, SpriteVerts(0)
    
    
    ' Set up and draw the Tiles across the top
    For i = 1 To iWidth - 2
        SetUpGeometry i, YPos, 32, 0, 96, 96
        D3DVertexBuffer8SetData VertBuffer, VertSize * 4 * i, VertSize * 4, 0, SpriteVerts(0)
    Next i
    
    ' Set up and draw the upper right corner
    SetUpGeometry iWidth - 1, YPos, 64, 0, 96, 96
    D3DVertexBuffer8SetData VertBuffer, VertSize * 4 * (iWidth - 1), VertSize * 4, 0, SpriteVerts(0)

    DrawVertBuffer "TextBox", iWidth

    ' Draw the middle row by row
    For i = 1 To iHeight - 1
    
        ' Set up and draw the far left side in this row
        SetUpGeometry 0, i + YPos, 0, 32, 96, 96
        D3DVertexBuffer8SetData VertBuffer, 0, VertSize * 4, 0, SpriteVerts(0)

        
        ' Set up and draw the The middle boxes
        For j = 1 To iWidth - 2
        
            SetUpGeometry j, i + YPos, 32, 32, 96, 96
            D3DVertexBuffer8SetData VertBuffer, VertSize * 4 * j, VertSize * 4, 0, SpriteVerts(0)
            
        Next j
    
        ' Set up and draw the far right side in this row
        SetUpGeometry iWidth - 1, i + YPos, 64, 32, 96, 96
        D3DVertexBuffer8SetData VertBuffer, VertSize * 4 * (iWidth - 1), VertSize * 4, 0, SpriteVerts(0)

        DrawVertBuffer "TextBox", iWidth
        
    Next i
    
    ' Set up and draw the bottom left corner
    SetUpGeometry 0, iHeight + YPos, 0, 64, 96, 96
    D3DVertexBuffer8SetData VertBuffer, 0, VertSize * 4, 0, SpriteVerts(0)

    ' Set up and draw the Tiles across the bottom
    For i = 1 To iWidth - 2
    
        SetUpGeometry i, iHeight + YPos, 32, 64, 96, 96
        D3DVertexBuffer8SetData VertBuffer, VertSize * 4 * i, VertSize * 4, 0, SpriteVerts(0)

    Next i
    
    ' Set up and draw the bottom right corner
    SetUpGeometry iWidth - 1, iHeight + YPos, 64, 64, 96, 96
    D3DVertexBuffer8SetData VertBuffer, VertSize * 4 * (iWidth - 1), VertSize * 4, 0, SpriteVerts(0)

    DrawVertBuffer "TextBox", iWidth
    
    ResetBuffer
        
End Sub

' Draws a textbox with text in it
Public Sub DrawTextBox(sText As String, lColor As Long)
    Dim sTemp As String
    Dim i As Long
    Dim s As String
    Dim ypos2 As Long
    
    Dim nLen As Integer
    Dim tmpInteger As Integer
    
    nLen = Len(sText)
    
    tmpInteger = nLen / (RESX / 5)
    
    ' Draw the box
    DrawBox
   
   ' Draw the text line by line
    ypos2 = 16
        
    sTemp = Left$(sText, RESX / 5)
        
    PlotText sTemp, 18, ypos2 + 2, RGB(255, 255, 255)
    PlotText sTemp, 16, ypos2, lColor
        
    ypos2 = ypos2 + 18
    
    For i = 1 To tmpInteger

        sTemp = Mid$(sText, i * (RESX / 5), (RESX / 5))
        
        PlotText sTemp, 18, ypos2 + 2, RGB(255, 255, 255)
        PlotText sTemp, 16, ypos2, lColor
        
        ypos2 = ypos2 + 18
    Next i
    
End Sub

' Sets up a light
Public Sub SetupLights(Index As Integer, Optional lType As CONST_D3DLIGHTTYPE = D3DLIGHT_DIRECTIONAL)

Dim Mtrl As D3DMATERIAL8, Col As D3DCOLORVALUE

' If out of range exit
If Index < 0 Then Exit Sub

    ' Set normal color reflections
    Col.a = 1
    Col.r = 1
    Col.g = 1
    Col.b = 1
    Mtrl.Ambient = Col
    Mtrl.diffuse = Col

    ' Set the material
    dev.SetMaterial Mtrl
    
    ' If not already made up, make it
    If UBound(Lights()) <= Index Then
        ReDim Preserve Lights(Index + 1) As D3DLIGHT8
    End If
    
    ' Change the type
    Lights(Index).Type = lType
    
    ' Set up location and data
    Select Case lType
        Case D3DLIGHT_DIRECTIONAL

        Case D3DLIGHT_SPOT
            Lights(Index).position = MakeVector(0, 0, 4)
            Lights(Index).Range = 100
            Lights(Index).Direction = MakeVector(0, 0, -1) 'Along the X axis
            Lights(Index).Theta = 30 * (3.14 / 180) 'Inner Cone
            Lights(Index).Phi = 50 * (3.14 / 180) 'Outer Cone
            Lights(Index).Attenuation1 = 0.05
        Case D3DLIGHT_POINT
            Lights(Index).position = MakeVector(0, 0, 1.5)
            Lights(Index).Direction = MakeVector(0, 0, 1)
            Lights(Index).Range = 4
            Lights(Index).Attenuation1 = 0.2
            'Lights(Index).Attenuation0 = 0.3
            Lights(Index).Attenuation2 = 0.2
    End Select
    
    Lights(Index).diffuse.b = 1
    Lights(Index).diffuse.g = 1
    Lights(Index).diffuse.r = 1

    
    ' Set the light
    dev.SetLight Index, Lights(Index)

    ' Enable it
    dev.LightEnable Index, 1

End Sub

' Move a light (Screen X,Y)
Public Sub MoveLight(PosX As Double, PosY As Double, Index As Integer)

    ' Select type and update position
    Select Case Lights(Index).Type
        Case D3DLIGHT_POINT
            Lights(Index).position.x = 5 - (PosX / TILEWIDTH + 0.5) * changeX
            Lights(Index).position.y = 5 - (PosY / TILEHEIGHT + 0.5) * changeY
        Case D3DLIGHT_SPOT
            Lights(Index).Direction.x = 5 - (PosX / TILEWIDTH + 1) * changeX
            Lights(Index).Direction.y = 5 - (PosY / TILEHEIGHT + 1) * changeY
    End Select

    ' Set it
    dev.SetLight Index, Lights(Index)

End Sub

' Enable/Disable Light
Public Sub EnableLights(Index As Integer, bEnabled As Boolean)
    dev.LightEnable Index, bEnabled
End Sub

' Make it a D3DVECTOR
Private Function MakeVector(x As Single, y As Single, Z As Single) As D3DVECTOR
    MakeVector.x = x
    MakeVector.y = y
    MakeVector.Z = Z
End Function

' Turn On/Off fog
Public Sub SetFog(bTurnOn As Boolean)

    With dev
        If bTurnOn Then
            .SetRenderState D3DRS_FOGENABLE, 1 ' True
            .SetRenderState D3DRS_FOGCOLOR, FOG_COLOR
            .SetRenderState D3DRS_FOGTABLEMODE, D3DFOG_NONE
            .SetRenderState D3DRS_FOGVERTEXMODE, D3DFOG_LINEAR
            .SetRenderState D3DRS_RANGEFOGENABLE, 0
            .SetRenderState D3DRS_FOGSTART, -1035468800
            .SetRenderState D3DRS_FOGEND, 1120014848
        Else
            .SetRenderState D3DRS_FOGENABLE, 0
        End If
    
    End With
        
End Sub

' Move a light to fallow a NPC
Public Sub MoveLight_NPC(Char As NPC_Character, MapOffsetX As Long, MapOffsetY As Long, MapWidth As Long, MapHeight As Long)
Dim CharX As Long, CharY As Long
Dim Index As Integer

    Index = Char.LightID
    
    ' Its not set right
    If Not Lights(Index).Type = D3DLIGHT_POINT Then Exit Sub

    CharX = Char.WorldX
    CharY = Char.WorldY
    
    ' Find Screen Coord
    If MapOffsetX > MapWidth - RESX Then
        If CharX > MapOffsetX Then
            CharX = CharX - MapOffsetX
        Else
            CharX = CharX + (MapWidth - MapOffsetX)
        End If
    Else
        CharX = CharX - MapOffsetX
    End If

    If MapOffsetY > MapHeight - RESY Then
        If CharY > MapOffsetY Then
            CharY = CharY - MapOffsetY
        Else
            CharY = CharY + (MapHeight - MapOffsetY)
        End If
    Else
        CharY = CharY - MapOffsetY
    End If
    
    ' If not visible, disable
    If CharX < 0 Or CharY < 0 Or CharX > RESX Or CharY > RESY Then
        dev.LightEnable Index, 0
    Else
        dev.LightEnable Index, 1
    End If

    ' Update its position
    Lights(Index).position.x = 5 - (CharX / TILEWIDTH + 0.5) * changeX
    Lights(Index).position.y = 5 - (CharY / TILEHEIGHT + 0.5) * changeY

    ' Set it
    dev.SetLight Index, Lights(Index)
    
End Sub

' Same as NPCs Move light, excpet for Char Class
Public Sub MoveLight_Char(Char As Character, MapOffsetX As Long, MapOffsetY As Long, MapWidth As Long, MapHeight As Long)
Dim CharX As Long, CharY As Long
Dim Index As Integer

Index = Char.LightID

CharX = Char.WorldX
CharY = Char.WorldY
    
    If MapOffsetX > MapWidth - RESX Then
        If CharX > MapOffsetX Then
            CharX = CharX - MapOffsetX
        Else
            CharX = CharX + (MapWidth - MapOffsetX)
        End If
    Else
        CharX = CharX - MapOffsetX
    End If

    If MapOffsetY > MapHeight - RESY Then
        If CharY > MapOffsetY Then
            CharY = CharY - MapOffsetY
        Else
            CharY = CharY + (MapHeight - MapOffsetY)
        End If
    Else
        CharY = CharY - MapOffsetY
    End If
    
    If CharX < 0 Or CharY < 0 Or CharX > RESX Or CharY > RESY Then
        dev.LightEnable Index, 0
    Else
        dev.LightEnable Index, 1
    End If

    Lights(Index).position.x = 5 - (CharX / TILEWIDTH + 0.5) * changeX
    Lights(Index).position.y = 5 - (CharY / TILEHEIGHT + 0.5) * changeY

    dev.SetLight Index, Lights(Index)
    
End Sub

' Draws the bottom map
Private Sub DrawMapBottom(tmpMap As clsMap)
Dim x As Long
Dim y As Long
Dim lx As Long, ly As Long
Dim tmpTile As Integer
Dim XStart As Integer
Dim YStart As Integer
Dim lOX As Long
Dim lOY As Long
Dim OffY As Long, OffX As Long
Dim CurrentX As Integer
Dim CurrentY As Integer
Dim tsHeight As Long, tsWidth As Long

    ' Get pixel Height and Width of map
    tsWidth = tmpMap.TileSetWidth * TILEWIDTH
    tsHeight = tmpMap.TileSetHeight * TILEHEIGHT

    ' determine the first (top left) tile to draw
    XStart = tmpMap.MapOffsetX \ TILEWIDTH
    YStart = tmpMap.MapOffsetY \ TILEWIDTH
    
    ' determine the pixel offset of the first tile
    lOX = (tmpMap.MapOffsetX) Mod TILEWIDTH: lOX = -lOX
    lOY = (tmpMap.MapOffsetY) Mod TILEHEIGHT: lOY = -lOY

    For y = 0 To RESY \ TILEHEIGHT
        For x = 0 To RESX \ TILEWIDTH + 1
            
            ' Get the id for the tile
            CurrentX = XStart + x
            CurrentY = YStart + y
            
            ' Check if it is round
            If tmpMap.IsRound Then

                ' If it is check if next tile must be wrapped from the other side

                If CurrentX > tmpMap.Width - 1 Then
                    CurrentX = XStart + x - tmpMap.Width
                ElseIf CurrentX < 0 Then
                    CurrentX = tmpMap.Width + CurrentX
                End If
            
                If CurrentY > tmpMap.Height - 1 Then
                    CurrentY = YStart + y - tmpMap.Height
                ElseIf CurrentY < 0 Then
                    CurrentY = tmpMap.Height + CurrentY
                End If
            Else
                If CurrentY > tmpMap.Height - 1 Then
                    CurrentY = tmpMap.Height - 1
                End If
                
                If CurrentX > tmpMap.Width - 1 Then
                    CurrentX = tmpMap.Width - 1
                End If
                
            End If
            
            ' Get the tile's ID number
            tmpTile = tmpMap.GetTileBottom(CurrentX, CurrentY)
            
            ' Set the destination offset
            lx = x * TILEWIDTH + lOX
            ly = y * TILEHEIGHT + lOY
            
            ' Set the source offset
            OffX = tmpMap.GetTileOffsetX(tmpTile)
            OffY = tmpMap.GetTileOffsetY(tmpTile)
            
            SetUpGeometry lx / TILEWIDTH, ly / TILEHEIGHT, OffX, OffY, tsWidth, tsHeight
            
            D3DVertexBuffer8SetData VertBuffer, VertSize * 4 * x, VertSize * 4, 0, SpriteVerts(0)

        Next x
        DrawVertBuffer (tmpMap.SurfaceName)
    Next y
    
End Sub

Private Sub DrawVertBuffer(SurfaceName As String, Optional Width As Long = -1)
Dim NumofTriangles As Long
    
    If Width = -1 Then
        NumofTriangles = (RESX / TILEWIDTH + 1) * 4 - 2
    Else
        NumofTriangles = (Width) * 4 - 2
    End If

    With dev
    
        .SetStreamSource 0, VertBuffer, VertSize
        
        'Set the Sprite texture on the device
        .SetTexture 0, oSurfaces(SurfaceName)
                                                  
        'Draw the 2 polygons that make up the Sprite
        .DrawPrimitive D3DPT_TRIANGLESTRIP, 0, NumofTriangles
  
    End With

End Sub

' Draws the middle map set
Private Sub DrawMapMiddle(tmpMap As clsMap)
Dim x As Long
Dim y As Long
Dim lx As Long, ly As Long
Dim tmpTile As Integer
Dim XStart As Integer
Dim YStart As Integer
Dim lOX As Long
Dim lOY As Long
Dim OffY As Long, OffX As Long
Dim CurrentX As Integer
Dim CurrentY As Integer
Dim tsHeight As Long, tsWidth As Long
Dim i As Integer

    ' Get pixel Height and Width of map
    tsWidth = tmpMap.TileSetWidth * TILEWIDTH
    tsHeight = tmpMap.TileSetHeight * TILEHEIGHT

    ' determine the first (top left) tile to draw
    XStart = tmpMap.MapOffsetX \ TILEWIDTH
    YStart = tmpMap.MapOffsetY \ TILEWIDTH
    
    ' determine the pixel offset of the first tile
    lOX = (tmpMap.MapOffsetX) Mod TILEWIDTH: lOX = -lOX
    lOY = (tmpMap.MapOffsetY) Mod TILEHEIGHT: lOY = -lOY

    For y = 0 To RESY \ TILEHEIGHT
        For x = 0 To RESX \ TILEWIDTH
            
            ' Get the id for the tile
            CurrentX = XStart + x
            CurrentY = YStart + y
            
            ' Check if it is round
            If tmpMap.IsRound Then

                ' If it is check if next tile must be wrapped from the other side

                If CurrentX > tmpMap.Width - 1 Then
                    CurrentX = XStart + x - tmpMap.Width
                ElseIf CurrentX < 0 Then
                    CurrentX = tmpMap.Width + CurrentX
                End If
            
                If CurrentY > tmpMap.Height - 1 Then
                    CurrentY = YStart + y - tmpMap.Height
                ElseIf CurrentY < 0 Then
                    CurrentY = tmpMap.Height + CurrentY
                End If
            Else
                If CurrentY > tmpMap.Height - 1 Then
                    CurrentY = tmpMap.Height - 1
                End If
                
                If CurrentX > tmpMap.Width - 1 Then
                    CurrentX = tmpMap.Width - 1
                End If
                
            End If
            
            ' Get the tile's ID number
            tmpTile = tmpMap.GetTileMiddle(CurrentX, CurrentY)
            
            ' If its not clear draw it
            If Not tmpTile = 0 Then
                ' Set the destination offset
                lx = x * TILEWIDTH + lOX
                ly = y * TILEHEIGHT + lOY
            
                    ' Set the source offset
                OffX = tmpMap.GetTileOffsetX(tmpTile)
                OffY = tmpMap.GetTileOffsetY(tmpTile)
            
                ' Set up the the square we are blitting from and to
                SetUpGeometry lx / TILEWIDTH, ly / TILEHEIGHT, OffX, OffY, tsWidth, tsHeight
            
                ' Blt the tile
                DrawSurface3D (tmpMap.SurfaceName)
                
            End If
                        
        Next x
    Next y

End Sub

' Draws the top map
Private Sub DrawMapTop(tmpMap As clsMap)
Dim x As Long
Dim y As Long
Dim lx As Long, ly As Long
Dim tmpTile As Integer
Dim XStart As Integer
Dim YStart As Integer
Dim lOX As Long
Dim lOY As Long
Dim OffY As Long, OffX As Long
Dim CurrentX As Integer
Dim CurrentY As Integer
Dim tsHeight As Long, tsWidth As Long

    ' Get the Pixel Height and Width of the map
    tsWidth = tmpMap.TileSetWidth * TILEWIDTH
    tsHeight = tmpMap.TileSetHeight * TILEHEIGHT

    ' determine the first (top left) tile to draw
    XStart = tmpMap.MapOffsetX \ TILEWIDTH
    YStart = tmpMap.MapOffsetY \ TILEWIDTH
    
    ' determine the pixel offset of the first tile
    lOX = (tmpMap.MapOffsetX) Mod TILEWIDTH: lOX = -lOX
    lOY = (tmpMap.MapOffsetY) Mod TILEHEIGHT: lOY = -lOY

    For y = 0 To RESY \ TILEHEIGHT
        For x = 0 To RESX \ TILEWIDTH
            
            ' Get the id for the tile
            CurrentX = XStart + x
            CurrentY = YStart + y
            
            ' Check if it is round
            If tmpMap.IsRound Then

                ' If it is check if next tile must be wrapped from the other side

                If CurrentX > tmpMap.Width - 1 Then
                    CurrentX = XStart + x - tmpMap.Width
                ElseIf CurrentX < 0 Then
                    CurrentX = tmpMap.Width + CurrentX
                End If
            
                If CurrentY > tmpMap.Height - 1 Then
                    CurrentY = YStart + y - tmpMap.Height
                ElseIf CurrentY < 0 Then
                    CurrentY = tmpMap.Height + CurrentY
                End If
            Else
                If CurrentY > tmpMap.Height - 1 Then
                    CurrentY = tmpMap.Height - 1
                End If
                
                If CurrentX > tmpMap.Width - 1 Then
                    CurrentX = tmpMap.Width - 1
                End If
                
            End If
            
            ' Get the tile's ID number
            tmpTile = tmpMap.GetTileTop(CurrentX, CurrentY)
            
            ' If its not clear draw it
            If Not tmpTile = 0 Then
                ' Set the destination offset
                lx = x * TILEWIDTH + lOX
                ly = y * TILEHEIGHT + lOY
            
                    ' Set the source offset
                OffX = tmpMap.GetTileOffsetX(tmpTile)
                OffY = tmpMap.GetTileOffsetY(tmpTile)
            
                ' Set up the the square we are blitting from and to
                SetUpGeometry lx / TILEWIDTH, ly / TILEHEIGHT, OffX, OffY, tsWidth, tsHeight
            
                ' Blt the tile
                DrawSurface3D (tmpMap.SurfaceName)
                
            End If
        Next x
    Next y

End Sub

' Turn off all lights
Public Sub TurnOffLights()
Dim i As Integer

    For i = 0 To NumberOfLights - 1
        dev.LightEnable i, 0
    Next i
    
End Sub

' Move a stationary light with respect to the view screen
Public Sub MoveLight_Map(ByVal PosX As Long, ByVal PosY As Long, Index As Integer, MapOffsetX As Long, MapOffsetY As Long, MapWidth As Long, MapHeight As Long)
    
    ' Find its relative position
    If MapOffsetX > MapWidth - RESX Then
        If PosX > MapOffsetX Then
            PosX = PosX - MapOffsetX
        Else
            PosX = PosX + (MapWidth - MapOffsetX)
        End If
    Else
        PosX = PosX - MapOffsetX
    End If

    If MapOffsetY > MapHeight - RESY Then
        If PosY > MapOffsetY Then
            PosY = PosY - MapOffsetY
        Else
            PosY = PosY + (MapHeight - MapOffsetY)
        End If
    Else
        PosY = PosY - MapOffsetY
    End If
    
    ' If not visible turn off
    If PosX < 0 Or PosY < 0 Or PosX > RESX Or PosY > RESY Then
        dev.LightEnable Index, 0
    Else
        dev.LightEnable Index, 1
    End If

    ' Update position
    Lights(Index).position.x = 5 - (PosX / TILEWIDTH + 0.5) * changeX
    Lights(Index).position.y = 5 - (PosY / TILEHEIGHT + 0.5) * changeY

    ' Set it
    dev.SetLight Index, Lights(Index)
    
End Sub

' will be for text with a format specification
' Syntax
' <BOLD> - bold until told not to
' <NORMAL> - normal text
' <ITALIC> - in italics
' <COLOR IIJJKK> - where the II is the red component, JJ is the green, KK is the blue
' <POS X,Y> - where X,Y are in percents of the screen
' <FULLSCREEN> - Fullscreen Boxthis must be first command unless dim is used, or
'                if <INPUTBOX ...> is used
' <DIM X,Y> - Dimensions of the textbox in percent, can be used anywhere in code
' <FONT font> - with font being replaced with the name of the new font
' <SIZE X> - size of font; x is size
' <SETMAXWIDTH X> - Where X is the max width of the text (percent of screen)
' <INPUTBOX X1,Y1 X2,Y2 ...> - Tells us that it is input box, make this the first command
'                              X1,Y1 are coords of first selection (percent) etc...
' <CURSOR X>  - X is the index of the current selection make last command
' <CURSOROFFSET X> - where X is number of selections to be incremented if user presses left/right
'                   does not have any effect on this procedure, it is taken card of in the map class
Public Sub DrawFormattedBox(sString As String)
Dim MainString() As String
Dim lkString() As String
Dim cmdString() As String
Dim tmpString() As String
Dim NumOfCommands As Integer
Dim FontDesc As IFont
Dim sLen As Integer         ' the strings length
Dim cLen As Integer         ' the commands length
Dim i As Integer
Dim x As Long, y As Long
Dim SizeX As Long, SizeY As Long
Dim lColor As Long
Dim iMaxWidth As Integer
Dim CursorX() As Integer
Dim CursorY() As Integer
    
    tColor = &HFFFFFF
    
    iMaxWidth = RESX
    
    MainString = Split(sString, "<")
    
    NumOfCommands = UBound(MainString())
    
    lkString = Split(MainString(1), ">")
    
    If RTrim(lkString(0)) = "FULLSCREEN" Then
        SizeX = 100
        SizeY = 100
    Else
        cmdString = Split(lkString(0), " ")
        tmpString = Split(cmdString(1), ",")
        SizeX = Val(tmpString(0))
        SizeY = Val(tmpString(1))
    End If
        
    DrawBox SizeY, SizeX
        
    For i = 2 To NumOfCommands
        If Not RTrim$(MainString(i)) = "" Then
        
            lkString() = Split(MainString(i), ">")
        
            Select Case lkString(0)
                Case "BOLD"
                    fnt.Bold = True
                Case "NORMAL"
                    fnt.Bold = False
                    fnt.Italic = False
                Case "ITALIC"
                    fnt.Italic = False
                Case Else
                    cmdString = Split(lkString(0), " ")
                    Select Case cmdString(0)
                        Case "INPUTBOX"
                            Dim j As Integer
                    
                            ReDim CursorX(UBound(cmdString) - 1)
                            ReDim CursorY(UBound(cmdString) - 1)
                    
                            For j = 1 To UBound(cmdString)
                                tmpString = Split(cmdString(j), ",")
                    
                                CursorX(j - 1) = Val(tmpString(0))
                                CursorY(j - 1) = Val(tmpString(1))
                            Next j
                        
                        Case "CURSOR"
                            Dim tmpIndex As Integer
                        
                            tmpIndex = Val(cmdString(1))
                        
                            'If tmpIndex > UBound(CursorX()) Then
                            '    tmpIndex = UBound(CursorX()) - 1
                            'ElseIf tmpIndex < 0 Then
                            '    tmpIndex = 0
                            'End If
                            
                            DrawSprite CurrentCustomFont, (RESX / 100) * CursorX(tmpIndex) - TILEWIDTH, CursorY(tmpIndex) * (RESY / 100) - TILEHEIGHT * 0.5, 256 - TILEWIDTH, 128 - TILEHEIGHT, 256, 128

                        Case "COLOR"
                        Case "POS"
                            tmpString = Split(cmdString(1), ",")
                            x = Val(tmpString(0)) / 100 * RESX
                            y = Val(tmpString(1)) / 100 * RESY
                        Case "FONT"
                            fnt.Name = cmdString(1)
                        Case "SETMAXWIDTH"
                            iMaxWidth = Val(cmdString(1)) * RESX / 100
                        Case "SIZE"
                            fnt.Size = Val(cmdString(1))
                        Case "DIM"
                            tmpString = Split(cmdString(1), ",")
                            SizeX = Val(tmpString(0))
                            SizeY = Val(tmpString(1))
                            DrawBox SizeY, SizeX
                    End Select
                
            End Select
        
        'Set FontDesc = fnt
        'Set D3DFont = d3dx.CreateFont(dev, FontDesc.hFont)
        'Set FontDesc = Nothing
        
            If LTrim(lkString(1)) <> "" Then
                PlotCustomFont lkString(1), x, y, fnt.Size, fnt.Size, iMaxWidth
                x = x + Len(lkString(1)) * fnt.Size * 0.75
            End If
        End If
    Next i

End Sub

' Code from DX4VB
' original availible at http://64.23.12.52/Tutorials/DirectX8/GR_Lesson06.htm
Private Sub PlotCustomFont(strText As String, StartX As Long, StartY As Long, Height As Integer, Width As Integer, Optional MaxX As Integer = 0)
Dim i As Integer, j As Integer
Dim CharX As Integer, CharY As Integer
Dim Char As String '//The current Character of the word
Dim CurrentX As Integer
Dim CurrentY As Integer
Dim LinearEntry As Integer 'Without going into 2D entries, just work it out if it were a line
Dim Words() As String
Dim NumOfWords As Integer
Dim iMaxX As Integer

    If StartX >= MaxX Then Exit Sub
    If Len(strText) = 0 Then Exit Sub

    CurrentX = StartX
    CurrentY = StartY

    If MaxX = 0 Then
        iMaxX = RESX
    Else
        iMaxX = MaxX
    End If
    
    Words = Split(strText, " ")
    
    NumOfWords = UBound(Words())
    
    For i = 0 To NumOfWords

        If CurrentX + (Len(Words(i)) * Width) >= iMaxX - Width Then
            CurrentX = StartX
            CurrentY = CurrentY + Height
        End If
        
        If CurrentY >= RESY Then Exit Sub
        
        For j = 1 To Len(Words(i))
    
            Char = Mid$(Words(i), j, 1)
                        
            If Asc(Char) >= 65 And Asc(Char) <= 90 Then
                LinearEntry = Asc(Char) - 65
            ElseIf Asc(Char) >= 97 And Asc(Char) <= 122 Then
                LinearEntry = Asc(Char) - 71
            ElseIf Asc(Char) >= 48 And Asc(Char) <= 57 Then
                LinearEntry = Asc(Char) + 4
            ElseIf Char = " " Then
                'Space
                LinearEntry = 63
                        
            ElseIf Char = "." Then
                'Full stop
                LinearEntry = 62
                            
            ElseIf Char = ";" Then
                'Semi colon
                LinearEntry = 66
        
            ElseIf Char = ":" Then
                LinearEntry = 67
                            
            ElseIf Char = "/" Then
                'Forward slash
                LinearEntry = 64
                            
            ElseIf Char = "," Then
                'Guess what; its a comma..
                LinearEntry = 65
                            
            End If
                
            'We now need to process the actual coordinates.
            If LinearEntry <= 15 Then
                CharY = 0
                CharX = LinearEntry
            End If
                        
            If LinearEntry >= 16 And LinearEntry <= 31 Then
                CharY = 1
                CharX = LinearEntry - 16
            End If
                        
            If LinearEntry >= 32 And LinearEntry <= 47 Then
                CharY = 2
                CharX = LinearEntry - 32
            End If
                        
            If LinearEntry >= 48 And LinearEntry <= 63 Then
                CharY = 3
                CharX = LinearEntry - 48
            End If
                        
            If LinearEntry >= 64 And LinearEntry <= 79 Then
                CharY = 4
                CharX = LinearEntry - 64
            End If
    
            SpriteVerts(0).x = 5 - (CurrentX + Width) * changeX / TILEWIDTH
            SpriteVerts(0).y = 5 - CurrentY * changeY / TILEHEIGHT
            SpriteVerts(0).tu = (1 / 16) * (CharX + 1)
            SpriteVerts(0).tv = (1 / 8) * CharY
             
            SpriteVerts(1).x = 5 - (CurrentX + Width) * changeX / TILEWIDTH
            SpriteVerts(1).y = 5 - (CurrentY + Height) * changeY / TILEHEIGHT
            SpriteVerts(1).tu = (1 / 16) * (CharX + 1)
            SpriteVerts(1).tv = (1 / 8) * (CharY + 1)
                
            SpriteVerts(2).x = 5 - CurrentX * changeX / TILEWIDTH
            SpriteVerts(2).y = 5 - CurrentY * changeY / TILEHEIGHT
            SpriteVerts(2).tu = (1 / 16) * CharX
            SpriteVerts(2).tv = (1 / 8) * CharY
                
            SpriteVerts(3).x = 5 - CurrentX * changeX / TILEWIDTH
            SpriteVerts(3).y = 5 - (CurrentY + Height) * changeY / TILEHEIGHT
            SpriteVerts(3).tu = (1 / 16) * CharX
            SpriteVerts(3).tv = (1 / 8) * (CharY + 1)

            dev.SetTexture 0, oSurfaces(CurrentCustomFont)
            dev.DrawPrimitiveUP D3DPT_TRIANGLESTRIP, 2, SpriteVerts(0), Len(SpriteVerts(0))
    
            CurrentX = CurrentX + Width
        Next j
        
        If Not Len(Words(i)) = 0 Then
            CurrentX = CurrentX + Width
        End If
        
    Next i
End Sub

