VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "MapConvertor"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Tile data
Private TileArray() As Tile              ' This array holds our tile data

' Map Data
Public sFilename As String               ' The name of the file the map is stored in
Public Width As Integer                  ' How many tiles wide it is
Public Height As Integer                 ' How many tiles high it is
Public CharStartX As Integer             ' where does the character enter in the X-Plane
Public CharStartY As Integer             ' Where does the character enter in the Y-Plane
Public IsRound As Boolean                ' Does the map wrap around
Private LightsDesc() As LightPos         ' DEscription of stationary lights (holds position and ID)

' Tile Set data (surface data)
Public SurfaceName As String             ' The name of the tile set the map uses
Public TileSetHeight As Integer          ' The height of the tile set
Public TileSetWidth As Integer           ' The width of the tile set
Public TileColorKey As Long              ' Colorkey for tiles in format ARGB (black = &H FF000000)

Private NPCData() As New NPC_Character   ' Holds NPC Data


' save the map to a file
Public Function SaveMap() As Boolean
On Error Resume Next

    Dim lFile As Long
    Dim i As Integer, j As Integer
    Dim nLen As Byte
    Dim tmpString As String
    Dim NumOfNPCs As Byte

    ' Get the number of NPCS
    NumOfNPCs = UBound(NPCData())
        
    'Kill sFilename
    Kill sFilename
    
    ' Get destination address
    lFile = FreeFile
    
    ' opens the address for writinh
    Open sFilename For Binary As lFile
    
    On Error GoTo ErrorHandler
    
    ' Save Current map version (Save it to an integer because it is a const
    ' and may have a different size in the future, so want to keep same length)
    i = CurrentMapVersion
    Put #lFile, , i
    
    ' save the tile width of the map
    Put #lFile, , Width
    
    ' save the tile height of the map
    Put #lFile, , Height
    
    ' Save if the tile is round
    Put #lFile, , IsRound
    
    ' Save the name of the tile surface
    nLen = Len(SurfaceName)
    Put #lFile, , nLen
    Put #lFile, , SurfaceName
    
    ' Save the tile Surface's width and height
    Put #lFile, , TileSetWidth
    Put #lFile, , TileSetHeight
    
    ' Save the tile Surface's Colorkey
    Put #lFile, , TileColorKey
    
    ' Save the start position of characters that enter
    Put #lFile, , CharStartX
    Put #lFile, , CharStartY
    
    ' save the map data to the file
    For i = 0 To Width - 1
        For j = 0 To Height - 1
            ' Save the bitmap data
            Put #lFile, , TileArray(i, j).Bottom
            Put #lFile, , TileArray(i, j).Middle
            Put #lFile, , TileArray(i, j).Top
            
            ' Save the tile data
            Put #lFile, , TileArray(i, j).Data
            
            ' Save the string length
            nLen = Len(TileArray(i, j).EventString)
            Put #lFile, , nLen
            
            ' If equal to "" then don't even bother saving
            If nLen <> 0 Then
                Put #lFile, , TileArray(i, j).EventString
            End If
            
        Next j
    Next i
    
    ' Save number of NPCs
    Put #lFile, , NumOfNPCs
    
    ' Attach NPC's file name to the map
    For i = 0 To NumOfNPCs - 1
        
        ' Get the length of the filename
        tmpString = NPCData(i).sFilename
        nLen = Len(tmpString)
        
        ' Save the length and name
        Put #lFile, , nLen
        Put #lFile, , tmpString
    
    Next i
    
    ' Get the number of lights and save it
    nLen = UBound(LightsDesc())
    Put #lFile, , nLen
    
    ' Save their coords.  The Ids will be set at runtime
    For i = 0 To nLen - 1
        Put #lFile, , LightsDesc(i).TileX
        Put #lFile, , LightsDesc(i).TileY
    Next i
    
    ' closes the file
    Close lFile
    
    ' Return True since it was successful
    SaveMap = True
    Exit Function

ErrorHandler:
    
    ' Save failed, return false
    SaveMap = False

End Function

' Load the map from a file
Public Function ConvertMap() As Boolean
    Dim lFile As Long
    Dim i As Integer

    lFile = FreeFile
    
    ' Open the file
    Open sFilename For Binary As lFile
    
    ' Get the version of the map and check it
    Get #lFile, , i
    
    Close lFile
    
    If i = CurrentMapVersion Then
        MsgBox "Already correct version!"
        Exit Function
    Else
        Select Case i
            Case 1
                ConvertMap1
        End Select
    End If

    SaveMap

End Function

Private Sub Class_Initialize()
    
    ' Size the arrays
    ReDim LightsDesc(0) As LightPos
    ReDim NPCData(0) As New NPC_Character
    
End Sub

Private Sub ConvertMap1()
    Dim lFile As Long
    Dim i As Integer, j As Integer
    Dim nLen As Byte
    Dim tmpString As String
    Dim NumOfNPCs As Byte
    Dim tmpByte As Byte
    Dim tmpInt As Integer
    Dim tmpLng As Long

    lFile = FreeFile
    
    ' Open the file
    Open sFilename For Binary As lFile
    
    ' Get the version of the map and check it
    Get #lFile, , i
    If i <> 1 Then
        MsgBox "Error"
        Close lFile
        Exit Sub
    End If
    
    ' Load the tile width of the map
    Get #lFile, , Width
    
    ' Load the tile height of the map
    Get #lFile, , Height
    
    ' Load if the Map is round
    Get #lFile, , IsRound
    
    ' Resize the tile array
    ReDim TileArray(Width, Height)
    
    ' Load the length of the surface name
    Get #lFile, , nLen
    SurfaceName = Space$(nLen)
    
    ' Load the surface name
    Get #lFile, , SurfaceName
    
    ' Load the surface width and height
    Get #lFile, , TileSetWidth
    Get #lFile, , TileSetHeight
    
    ' Load the tile set color key
    Get #lFile, , TileColorKey
        
    ' Load character start positions
    Get #lFile, , CharStartX
    Get #lFile, , CharStartY
    
    ' Load the map data to the file
    For i = 0 To Width - 1
        For j = 0 To Height - 1
        
            ' Load the bitmap data
            Get #lFile, , TileArray(i, j).Bottom
            Get #lFile, , TileArray(i, j).Middle
            Get #lFile, , TileArray(i, j).Top
            
            ' Load the properties of the tile
            Get #lFile, , tmpInt
            
            TileArray(i, j).Data = tmpInt
            
            ' GEt the length of its string
            Get #lFile, , nLen
            
            ' If zero, its not saved
            If nLen <> 0 Then
                TileArray(i, j).EventString = Space$(nLen)
                Get #lFile, , TileArray(i, j).EventString
            End If
        
        Next j
    Next i
    
    ' Get the number of NPCs
    Get #lFile, , NumOfNPCs
    
    ' Resize the NPC array
    ReDim NPCData(NumOfNPCs)
    
    ' Load all their filenames
    For i = 0 To NumOfNPCs - 1
    
        Get #lFile, , nLen
        tmpString = Space$(nLen)
    
        Get #lFile, , tmpString
    
        NPCData(i).sFilename = tmpString
    
    Next i
    
    ' Get the number of lights
    Get #lFile, , nLen
    
    ' Resize the array
    ReDim LightsDesc(nLen)
    
    ' Set their location
    For i = 0 To nLen - 1
        Get #lFile, , LightsDesc(i).TileX
        Get #lFile, , LightsDesc(i).TileY
    Next i
    
    ' closes the file
    Close lFile

End Sub
