VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "NPC_Character"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit

' The Character Data
Public sFilename As String                      ' The name of the file to store the Character data
Private CharName As String                      ' The name of our character
Public LightID As Integer                       ' Holds Id of Light associated with NPC

' We need to position variables for each so characters will not run over eachother
Private TileX As Integer, TileY As Integer      ' Holds the spot where the character is moving to

' Holds the status of sprite
Public WorldX As Long, WorldY As Long           ' Holds the world x,y Position
Private Status As Byte                          ' Holds the status of the character ( which way he/she is walking, talking...)

' Variables for the movement of the sprite
Private FrameCount As Byte                      ' What frame number are we on
Private CurrentFrame As Byte                    ' What sprite are we currently using

' Animation tile set (Surface)
Private TileSetName As String                   ' The name of the tiles that contains the characters sprites
Private TileSetWidth As Byte                    ' The number of tiles wide it is
Private TileSetHeight As Byte                   ' The number of tiles high it is

Public FrameOffsetX As Integer                  ' Offset of the characters frame
Public FrameOffsetY As Integer                  ' Offset of the characetrs frame
Private OffsetX As Integer                      ' The offset of the tile x-coord. (for the sprite)
Private OffsetY As Integer                      ' The offset of the tile y-coord. (for the sprite)
Private TalkStrings() As String                 ' Holds an array of strings, used for NPC talking
Private NumberofStrings As Integer              ' Number of talk strings
Private CurrentString As Integer                ' Current talk string
Public bActive As Boolean                       ' Is NPC active

' Get number of talk strings
Public Property Get NumOfStrings() As Integer
    NumOfStrings = NumberofStrings
End Property

' Add a talk string
Public Function AddTalkString(tmpString As String, Optional Index As Integer = 0) As Integer
    
    ' If new string index = 0, so make new one
    If Index = 0 Then
        
        ' Update number of strings
        NumberofStrings = NumberofStrings + 1
        ReDim Preserve TalkStrings(NumberofStrings)
        
        ' Set the string
        TalkStrings(NumberofStrings - 1) = tmpString
        
    ElseIf Index <= NumberofStrings Then ' Else set it
        TalkStrings(Index - 1) = tmpString
    End If
    
End Function

' Get a talk string
Public Function GetTalkString(Optional Index As Integer = 0) As String

    ' If 0 then getting current talk string and increment next one
    If Index = 0 Then
        GetTalkString = TalkStrings(CurrentString)

        ' Only increment if more availible
        If CurrentString < NumberofStrings - 1 Then
            CurrentString = CurrentString + 1
        End If
    ElseIf Index <= NumberofStrings Then ' Else get specific string
        GetTalkString = TalkStrings(Index - 1)
    End If
    
End Function

' Set the direction that the character is facing
Public Sub SetFaceDirection(Direction As Integer)
    OffsetY = Direction + FrameOffsetY - 1
End Sub

' Get the direction the character is facing
Public Function GetFaceDirection() As Integer
    GetFaceDirection = OffsetY + 1
End Function

' Get the x-offset of the next animation frame
Public Property Get iOffsetX() As Integer
    iOffsetX = OffsetX * TILEWIDTH
End Property

' Get the name of the tile set for the character
Public Property Get SurfaceName() As String
    SurfaceName = TileSetName
End Property

' Get the y-offset of the next animation frame
Public Property Get iOffsetY() As Integer
    iOffsetY = OffsetY * TILEHEIGHT
End Property

' Sets up the tile set
Public Sub InitTileSet(TSetName As String, TSetHeight As Byte, TSetWidth As Byte, TStartX As Byte, TStartY As Byte)
    
    ' Name of the Tileset (collection name)
    TileSetName = TSetName
    
    ' Height and width in tiles
    TileSetHeight = TSetHeight
    TileSetWidth = TSetWidth
    
    ' Offsets of the character within the bitmap, in tiles
    FrameOffsetX = TStartX
    FrameOffsetY = TStartY
    
    ' Updates the first sprite drawn
    OffsetX = TStartX
    OffsetY = TStartY
    
End Sub

' Initialize the character
Public Sub SetUpChar(Name As String, Optional XStart As Integer = 0, Optional YStart As Integer = 0)
    
    ' Set up the character data
    CharName = Name
    
    ' Set up the start position
    TileX = XStart
    TileY = YStart

    ' Set world start position
    WorldX = TILEHEIGHT * XStart
    WorldY = TILEHEIGHT * YStart
    
    bActive = True
    
End Sub

' Get the number of tile down in the surface
Public Property Get TilesHigh() As Byte
    TilesHigh = TileSetHeight
End Property

' Get the number of tiles across in the surface
Public Property Get TilesWide() As Byte
    TilesWide = TileSetWidth
End Property

' Gets the tile set name
Public Property Get TileName() As String
    TileName = TileSetName
End Property

' Gets the characters name
Public Property Get CharacterName() As String
    CharacterName = CharName
End Property

' Get the status of the Character
Public Property Get State() As Byte
    State = Status
End Property

' Get tile X-coord.
Public Function GetTileX() As Integer
    GetTileX = TileX
End Function

' Get tile Y-coord.
Public Function GetTileY() As Integer
    GetTileY = TileY
End Function

' Set tile X-coord.
Public Sub SetTileX(tmpX As Integer)
    TileX = tmpX
End Sub

' Set tile Y-coord.
Public Sub SetTileY(tmpY As Integer)
    TileY = tmpY
End Sub

' See if we need to change character sprites (for walking)
Private Sub Animate()
    
    FrameCount = FrameCount + 1
    
    ' If we have waited the amount of time needed do next frame
    If Not FrameCount < FRAMEWAIT Then
        
        ' If the current frame is the last one in the set, goto the first
        If Not CurrentFrame < MOVEFRAMES - 1 Then
            CurrentFrame = 0
        
        ' Else we add one to its counter
        Else
            CurrentFrame = CurrentFrame + 1
        End If
        
        FrameCount = 1
        
    End If
    
    ' Set the sprite X-coord
    OffsetX = CurrentFrame + FrameOffsetX
    
End Sub

' Starts the movement cycle
Public Sub MoveCharacter(Direction As Integer)
    
    ' Only move if not moving already or doing something
    If Status = 0 Then
        Select Case Direction
            Case MEUP
                TileY = TileY - 1
            Case MEDOWN
                TileY = TileY + 1
            Case MERIGHT
                TileX = TileX + 1
            Case MELEFT
                TileX = TileX - 1
        End Select
        
        ' Set the character as moving in that direction
        Status = Direction
    
    End If

End Sub

' If needed move the character
Public Sub DoCharEvents()
    
    Select Case Status
        Case MEUP ' Move the character up
            
            ' Set the sprites we want to use
            OffsetY = MEUP + FrameOffsetY - 1

            ' If the screen has moved we don't need to move the character
            
            ' Move characters world position
            WorldY = WorldY - PIXELSPEED
            
            ' Animate the character
            Animate
            
            ' If moved to new spot then move finished
            If WorldY Mod 32 = 0 Then Status = 0
        
        Case MEDOWN ' Move the character down

            ' Set the sprites we want to use
            OffsetY = MEDOWN + FrameOffsetY - 1
            
            ' Move characters world position
            WorldY = WorldY + PIXELSPEED
 
            ' Animate the character
            Animate
            
            ' If moved to new spot then move finished
            If WorldY Mod 32 = 0 Then Status = 0
        
        Case MERIGHT ' Move the character right
            
            ' Set the sprites we want to use
            OffsetY = MERIGHT + FrameOffsetY - 1

            ' Move characters world position
            WorldX = WorldX + PIXELSPEED
                
            ' Animate the character
            Animate
            
            ' If moved to new spot then move finished
            If WorldX Mod 32 = 0 Then Status = 0
        
        Case MELEFT ' Move the character left
            
            ' Set the sprites we want to use
            OffsetY = MELEFT + FrameOffsetY - 1
            
            ' Move characters world position
            WorldX = WorldX - PIXELSPEED
            
            ' Animate the character
            Animate
            
            ' If moved to new spot then move finished
            If WorldX Mod 32 = 0 Then Status = 0
        
        Case METALK
            ' Talk Function
            
    End Select
    
End Sub


' Saves the character data  (to be implemented later)
Public Function SaveCharacter() As Boolean

    Dim lFile As Long
    Dim i As Integer, j As Integer
    Dim nLen As Byte
    Dim tmpString As String
    
    'Kill sFilename
    
    ' Get destination address
    lFile = FreeFile
    
    ' opens the address for writinh
    Open CommonPath & "\" & sFilename For Binary As lFile
    
    On Error GoTo ErrorHandler
    
    nLen = Len(CharName)
    Put #lFile, , nLen
    Put #lFile, , CharName
    ' Save the name of the tile surface
    nLen = Len(TileSetName)
    Put #lFile, , nLen
    Put #lFile, , TileSetName
    
    ' Save the tile Surface's width and height
    Put #lFile, , TileSetWidth
    Put #lFile, , TileSetHeight
    
    Put #lFile, , FrameOffsetX
    Put #lFile, , FrameOffsetY

    Put #lFile, , TileX
    Put #lFile, , TileY
    
    Put #lFile, , NumberofStrings

    For i = 0 To (NumberofStrings - 1)
        ' Load the length of the file name
        tmpString = TalkStrings(i)
        
        nLen = Len(tmpString)
        Put #lFile, , nLen
        Put #lFile, , tmpString
        
    Next i
    
    ' closes the file
    Close lFile
    
    ' Return True since it was successful
    SaveCharacter = True
    Exit Function

ErrorHandler:
    
    ' Save failed, return false
    SaveCharacter = False
End Function

' Loads the character data  (to be implemented later)
Public Function LoadCharacter() As Boolean

    Dim lFile As Long
    Dim i As Integer, j As Integer
    Dim nLen As Byte
    Dim tmpString As String
    
    'Kill sFilename
    
    ' Get destination address
    lFile = FreeFile
    
    ' opens the address for writinh
    Open CommonPath & "\" & sFilename For Binary As lFile
    
    On Error GoTo ErrorHandler
    
    Get #lFile, , nLen
    CharName = Space$(nLen)
    Get #lFile, , CharName
    
    ' Save the name of the tile surface
    Get #lFile, , nLen
    
    TileSetName = Space$(nLen)
    Get #lFile, , TileSetName
    
    ' Save the tile Surface's width and height
    Get #lFile, , TileSetWidth
    Get #lFile, , TileSetHeight
    
    Get #lFile, , FrameOffsetX
    Get #lFile, , FrameOffsetY

    Get #lFile, , TileX
    Get #lFile, , TileY
    
    Get #lFile, , NumberofStrings
    
    ReDim TalkStrings(NumberofStrings)

    For i = 0 To (NumberofStrings - 1)
        ' Load the length of the file name
        
        Get #lFile, , nLen
         
        tmpString = Space$(nLen)

        Get #lFile, , tmpString
        
        TalkStrings(i) = tmpString
        
    Next i
    
    WorldX = TileX * TILEWIDTH
    WorldY = TileY * TILEHEIGHT
    
    OffsetX = FrameOffsetX
    OffsetY = FrameOffsetY
    
    bActive = True
    
    ' closes the file
    Close lFile
    
    ' Return True since it was successful
    LoadCharacter = True
    Exit Function

ErrorHandler:
    
    ' Save failed, return false
    LoadCharacter = False

End Function

' Check if the character is moving (not talking or standing)
Public Property Get IsMoving() As Boolean
    If Status = 1 Or Status = 2 Or Status = 3 Or Status = 4 Then
        IsMoving = True
    End If
End Property


