VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "SurfaceEdit"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Tile Set data (surface data)
Public TileSetHeight As Integer          ' The height of the tile set
Public TileSetWidth As Integer           ' The width of the tile set
Public TileColorKey As Long              ' Colorkey for tiles in format ARGB (black = &H FF000000)

' Edit mode features
Public TileSurfaceOffsetY As Integer     ' What is the start tile y-coord

Public DisplayWidth As Integer
Public DisplayHeight As Integer

Private TileOffsetX() As Long            ' The value of the tile data goes in the brackets and returns x pos
Private TileOffsetY() As Long            ' The value of the tile data goes in the brackets and returns y pos

' Direct Draw variables
Private dx As DirectX8                  ' The main DX object
Private SpriteVerts(3) As TLVERTEX      ' Vertex information for this sprite.
Private d3d As Direct3D8                ' The direct 3d object
Private dev As Direct3DDevice8          ' The main device
Private d3dx As D3DX8                   ' Helper library
Private oSurface As Direct3DTexture8
Private Light As D3DLIGHT8

Private matWorld As D3DMATRIX
Private matView As D3DMATRIX
Private matProj As D3DMATRIX
Public aHwnd As Long                    ' stores the window handle of our main window

  

Public Sub DrawMe()
    'The app is ready for rendering.
    With dev
                                    
        'Clear the back buffer
        .Clear 0, ByVal 0, D3DCLEAR_TARGET, 0, 1#, 0 '//Clear the screen black
            
        'Begin the 3d scene
        .BeginScene
    End With
    
        DrawMapSurface
        
    With dev
    
        'End the scene
        .EndScene
            
        'Draw the graphics to the front buffer.
        .Present ByVal 0&, ByVal 0&, 0, ByVal 0&
            
    End With

End Sub

' Draws the surface that holds all the different tiles (Used for editing)
Public Sub DrawMapSurface()
Dim x As Long
Dim y As Long
Dim lx As Long, ly As Long
Dim tmpTile As Integer
Dim OffY As Long, OffX As Long
Dim count As Integer
Dim tsHeight As Long, tsWidth As Long
Dim XMax As Integer
Dim YMax As Integer

    tsWidth = TileSetWidth * TILEWIDTH
    tsHeight = TileSetHeight * TILEHEIGHT
    count = 0

    If DisplayWidth <> 0 Then
        XMax = DisplayWidth - 1
        YMax = (TileSetHeight * TileSetWidth) / DisplayWidth
    Else
        XMax = TileSetWidth - 1
        YMax = TileSetHeight - TileSurfaceOffsetY - 1
    End If
   
    count = 0
    
    If TileSurfaceOffsetY > 0 Then
        count = TileSurfaceOffsetY * DisplayWidth
    End If
    
    For y = 0 To DisplayHeight
        For x = 0 To XMax
        
            tmpTile = count
            
            OffX = GetTileOffsetX(tmpTile)
            OffY = GetTileOffsetY(tmpTile)
            
            lx = x * TILEWIDTH
            ly = y * TILEHEIGHT
            
            SetUpGeometry x, y, OffX, OffY, tsWidth, tsHeight
            
            ' Blt the tile
            DrawSurface3D
            
            count = count + 1
            
        Next x
    Next y
End Sub

Public Sub InitD3D()

Dim DispMode As D3DDISPLAYMODE
Dim D3DWindow As D3DPRESENT_PARAMETERS
Dim Mtrl As D3DMATERIAL8, Col As D3DCOLORVALUE

    ' Initialize the devices
    Set dx = New DirectX8
    Set d3d = dx.Direct3DCreate()
    Set d3dx = New D3DX8

    ' We only want one backbuffer
    D3DWindow.BackBufferCount = 1

    ' Sets the target for drawing
    D3DWindow.hDeviceWindow = aHwnd


    'Get the current display mode
    d3d.GetAdapterDisplayMode D3DADAPTER_DEFAULT, DispMode

    'For windowed mode, we just discard any information instead of flipping it.
    D3DWindow.SwapEffect = D3DSWAPEFFECT_DISCARD
                        
    'Set windowed mode to true.
    D3DWindow.Windowed = 1
            
    ' Set up the format
    D3DWindow.BackBufferFormat = DispMode.Format

    ' Sets up the main device, we use software vertex processing, since it has little
    ' effect on performance( might need to change for slow CPU machines to hardware)
    Set dev = d3d.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, aHwnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, D3DWindow)

    ' Set up the flexible vertex format (Const in modConst.bas)
    dev.SetVertexShader FVF

    'Create the vertices for the Sprite
    Ambient = &HFFFFFF
    dev.SetRenderState D3DRS_AMBIENT, Ambient 'The ambient value is a hex-RRGGBB code
    
    Col.a = 1
    Col.r = 1
    Col.g = 1
    Col.b = 1
    Mtrl.Ambient = Col
    Mtrl.diffuse = Col

    dev.SetMaterial Mtrl
    
    Light.type = D3DLIGHT_POINT
    
    Light.position = MakeVector(0, 0, 1.5)
    Light.Direction = MakeVector(0, 0, 1)
    Light.Range = 4
    Light.Attenuation1 = 0.2
    'Light.Attenuation0 = 0.3
    Light.Attenuation2 = 0.2
    
    Light.diffuse.b = 1
    Light.diffuse.g = 1
    Light.diffuse.r = 1

    
    dev.SetLight 0, Light

    dev.LightEnable 0, 1
    
    ' Setup the matrixes
    D3DXMatrixIdentity matWorld ' World matrix

    ' View matrix
    D3DXMatrixLookAtLH matView, MakeVector(0, 0, 9), MakeVector(0, 0, 0), MakeVector(0, 1, 0)
    dev.SetTransform D3DTS_VIEW, matView

    ' Projection matrix
    D3DXMatrixPerspectiveFovLH matProj, 1, 1, 0.1, 500
    dev.SetTransform D3DTS_PROJECTION, matProj
    
    ' Make the normals of all the vertexes
    SpriteVerts(0).nz = 1
    SpriteVerts(1).nz = 1
    SpriteVerts(2).nz = 1
    SpriteVerts(3).nz = 1
    
    
End Sub
' Draws the surface with the current vertex information
Private Sub DrawSurface3D()
    
    With dev
        
        'Set the Sprite texture on the device
        .SetTexture 0, oSurface
                                                  
        'Draw the 2 polygons that make up the Sprite
        .DrawPrimitiveUP D3DPT_TRIANGLESTRIP, 2, SpriteVerts(0), Len(SpriteVerts(0))
  
    End With

End Sub

' Sets up the source and destination coordinates
Private Sub SetUpGeometry(ByVal StartX As Double, ByVal StartY As Double, ByVal tOffsetX As Double, ByVal tOffsety As Double, ByVal SizeX As Double, ByVal SizeY As Double)
Dim Cx As Double, Cy As Double
        ' .X is the X coordinate
        ' .Y is the Y coordinate
        '
        '   See figure for coord sys.  Notice 0,0 is in center
        '
        '   (4,4)
        '   |------------|
        '   |            |
        '   |    (0,0)   |
        '   |            |
        '   |____________|
        '                (-4,-4)
        '
        ' .tu is the source X, can only be 0 to 1.
        '      (Ex. if want halfway through a bitmap with 256 pixels wide, .tu = .5
        ' .tv is the source Y, can only be 0 to 1.

        Cx = 10 / DisplayWidth
        Cy = 10 / DisplayHeight

        SpriteVerts(0).x = 5 - (StartX + 1) * (Cx)
        SpriteVerts(0).y = 5 - StartY * Cy
        SpriteVerts(0).tu = (tOffsetX + TILEWIDTH) / SizeX
        SpriteVerts(0).tv = tOffsety / SizeY
             
        SpriteVerts(1).x = 5 - (StartX + 1) * Cx
        SpriteVerts(1).y = 5 - (StartY + 1) * Cy
        SpriteVerts(1).tu = (tOffsetX + TILEWIDTH) / SizeX
        SpriteVerts(1).tv = (tOffsety + TILEHEIGHT) / SizeY
                
        SpriteVerts(2).x = 5 - StartX * Cx
        SpriteVerts(2).y = 5 - StartY * Cy
        SpriteVerts(2).tu = tOffsetX / SizeX
        SpriteVerts(2).tv = tOffsety / SizeY
                
        SpriteVerts(3).x = 5 - StartX * Cx
        SpriteVerts(3).y = 5 - (StartY + 1) * Cy
        SpriteVerts(3).tu = tOffsetX / SizeX
        SpriteVerts(3).tv = (tOffsety + TILEHEIGHT) / SizeY
    
End Sub


Private Function MakeVector(x As Single, y As Single, Z As Single) As D3DVECTOR
    MakeVector.x = x
    MakeVector.y = y
    MakeVector.Z = Z
End Function

' Get the tile offsets from the tile set
Public Sub CreateTileSetOffset()
Dim i As Integer, j As Integer
Dim tmpVal As Integer
  
    ' Set the size of the offset tables
    ReDim TileOffsetY(TileSetHeight * TileSetWidth)
    ReDim TileOffsetX(TileSetWidth * TileSetHeight)
    
    ' Start at 0
    tmpVal = 0
    
    ' Set up the tile offsets
    For i = 0 To TileSetHeight - 1
        For j = 0 To TileSetWidth - 1
            
            TileOffsetX(tmpVal) = j * TILEWIDTH
            TileOffsetY(tmpVal) = i * TILEHEIGHT
            
            ' Increment the count
            tmpVal = tmpVal + 1
            
        Next j
    Next i
End Sub

' Get the tile's X offset from the tile surface ( Returns value)
Public Function GetTileOffsetX(i As Integer) As Long
    GetTileOffsetX = TileOffsetX(i)
End Function

' Get the tile's Y offset from the tile surface ( Returns value)
Public Function GetTileOffsetY(i As Integer) As Long
    GetTileOffsetY = TileOffsetY(i)
End Function

Public Function LoadSurface(sFilename As String)

    Set oSurface = d3dx.CreateTextureFromFileEx(dev, sFilename, D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED, D3DX_FILTER_POINT, D3DX_FILTER_POINT, ColorKeyValue, ByVal 0, ByVal 0)

End Function
Public Function TerminateD3D()
    
    Set oSurface = Nothing
    Set dev = Nothing
    Set d3d = Nothing
    Set dx = Nothing
End Function
