VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsMap"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Tile data
Private TileArray() As Tile              ' This array holds our tile data
Private TileOffsetX() As Long            ' The value of the tile data goes in the brackets and returns x pos
Private TileOffsetY() As Long            ' The value of the tile data goes in the brackets and returns y pos

' Map Data
Public sFilename As String               ' The name of the file the map is stored in
Public Width As Integer                  ' How many tiles wide it is
Public Height As Integer                 ' How many tiles high it is
Private OffsetX As Long                  ' The X offset of the map
Private OffsetY As Long                  ' The Y offset of the map
Public CharStartX As Integer             ' where does the character enter in the X-Plane
Public CharStartY As Integer             ' Where does the character enter in the Y-Plane
Public IsRound As Boolean                ' Does the map wrap around
Public IsSet As Boolean                  ' Is the map set up
Private LightsDesc() As LightPos         ' Description of stationary lights (holds position and ID)

' Tile Set data (surface data)
Public SurfaceName As String             ' The name of the tile set the map uses
Public TileSetHeight As Integer          ' The height of the tile set
Public TileSetWidth As Integer           ' The width of the tile set
Public TileColorKey As Long              ' Colorkey for tiles in format ARGB (black = &H FF000000)

' Debug Features
Public BottomVisible As Boolean          ' Is the bottom tile set visible
Public MiddleVisible As Boolean          ' Is the middle tile set visible
Public TopVisible As Boolean             ' Is the top tile set visible
Public SpecialVisible As Boolean         ' Is the Data tile set visible

' Characters that are in the Map
Private CharData() As New Character      ' Hold the character info
Private NPCData() As New NPC_Character   ' Holds NPC Data
Private ActiveChar As Integer            ' Which character controls the scroll of the screen
Private ActiveScripts() As String

' Textbox and InputBox variables
Private IsTalkBox As Boolean
Private CursorIndex As Integer
Private CursorMax As Integer
Private CursorOffset As Integer
Public Property Get LastSelection() As Integer
    LastSelection = CursorIndex
End Property

Public Sub DeleteScript(i As Integer)
Dim iSize As Integer

    iSize = UBound(ActiveScripts()) - 1
    
    If iSize < i Then Exit Sub
    
    If Not iSize = i Then
        ActiveScripts(i) = ActiveScripts(iSize)
    End If

    ReDim Preserve ActiveScripts(iSize)

End Sub

Public Sub AddScript(sEventString As String)
Dim iSize As Integer

    iSize = UBound(ActiveScripts())
    
    ReDim Preserve ActiveScripts(iSize + 1)
    ActiveScripts(iSize) = sEventString
    
End Sub

Public Sub CreateTerrainData()

    ReDim TerrainData(Width * TILEWIDTH / 8, Height * TILEHEIGHT / 8)
    
End Sub

' Adds a stationary light to the map
Public Sub AddLight(TileX As Integer, TileY As Integer)
Dim i As Integer
    
    ' If light already exists, exit
    If FindLightID(TileX, TileY) <> -1 Then Exit Sub
    
    ' Find out how many lights exist
    i = UBound(LightsDesc())
    
    ' Resize the array accordingly
    ReDim Preserve LightsDesc(i + 1)
    
    ' Set up new description
    With LightsDesc(i)
        .TileX = TileX
        .TileY = TileY
        .ID = 0
    End With
        
End Sub

' Returns the number of stationary lights
Public Function NumberOfLights() As Integer
    NumberOfLights = UBound(LightsDesc())
End Function

' Returns the number of characters
Public Function NumberOfCharacters() As Integer
    NumberOfCharacters = UBound(CharData())
End Function

' Returns the number of NPCs
Public Function NumberOfNPCs() As Integer
    NumberOfNPCs = UBound(NPCData())
End Function

' Finds the ID of a light at a specific tile, and returns it
Public Function FindLightID(TileX As Integer, TileY As Integer) As Integer
Dim i As Integer
Dim tmpLength As Integer

    ' How many lights do we have to search through
    tmpLength = UBound(LightsDesc()) - 1
    
    ' Loop through all until we find it
    For i = 0 To tmpLength
        
        ' The multilayered boolean testing is faster, since if X is false, it won't even test Y
        If LightsDesc(i).TileX = TileX Then
            If LightsDesc(i).TileY = TileY Then
                
                ' Found the light, Return it and exit
                FindLightID = i
                
                Exit Function
            End If
        End If
    Next i

    ' Returns -1 if not found
    FindLightID = -1

End Function

' Erases a light
Public Sub DeleteLight(TileX As Integer, TileY As Integer)
Dim i As Integer
Dim LightID As Integer
Dim tmpLight As LightPos
    
    ' Find the lights ID
    LightID = FindLightID(TileX, TileY)
    
    ' If it does not exist, exit
    If LightID < 0 Then Exit Sub
    
    ' Find total number of lights
    i = UBound(LightsDesc()) - 1
    
    If i < 0 Then Exit Sub
    
    ' If not last light, then exchange it with the last light
    If LightID <> i Then
        tmpLight = LightsDesc(i)
        LightsDesc(LightID) = tmpLight
    End If
    
    ' Just redim the array to not include the last light
    ReDim Preserve LightsDesc(i)
    
End Sub

' Gets the CharID on a specific tile (0 if no character)
Public Property Get GetCharOnTile(i As Integer, j As Integer) As Integer
    GetCharOnTile = TileArray(i, j).CharID
End Property

' Sets the Char/NPCs light
Public Sub SetLightIDChar(CharID As Integer, LightID As Integer, Optional IsNPC As Boolean = False)
    
    ' Check if it is an NPC or character
    If IsNPC Then
        NPCData(CharID).LightID = LightID
    Else
        CharData(CharID).LightID = LightID
    End If
    
End Sub

' Gets the Char/NPC light
Public Function GetLightID(CharID As Integer, Optional IsNPC As Boolean = False) As Integer
    
    ' Check if it is an NPC of Character
    If IsNPC Then
        GetLightID = NPCData(CharID).LightID
    Else
        GetLightID = CharData(CharID).LightID
    End If
    
End Function

' Get the TileX of a certain Light
Public Function GetLightTileX(ID As Integer) As Integer
    GetLightTileX = LightsDesc(ID).TileX
End Function

' Get the TileY of a certain Light
Public Function GetLightTileY(ID As Integer) As Integer
    GetLightTileY = LightsDesc(ID).TileY
End Function

' Add a talk string to an NPC
Public Sub AddTextStringNPC(Index As Integer, sTalkString As String)
    NPCData(Index).AddTalkString sTalkString
End Sub

' Check if animation is needed for the characters (And other stuff later)
Public Sub DoMapEvents(tmpEngine As D3D8Engine)
Dim i As Integer
Dim NumOfLights As Integer
Dim NumOfChars As Integer
Dim TotalWidth As Long
Dim TotalHeight As Long
Dim NumOfNPCs As Integer

    ' Get the number of NPCs/Lights/Chars
    NumOfNPCs = NumberOfNPCs
    NumOfLights = UBound(LightsDesc())
    NumOfChars = NumberOfCharacters

    TotalWidth = Width * TILEWIDTH
    TotalHeight = Height * TILEHEIGHT

    DecodeActiveEvents tmpEngine

    For i = 0 To NumOfChars - 1
        ' Check if the character is both moving and set up
        If CharData(i).IsMoving And CharData(i).IsSet Then
            
            ' If it is the active character we have to check to see if we want to scroll
            If i = ActiveChar Then
                CharData(i).DoCharEvents CheckScroll(CharData(i).State)
            Else
                CharData(i).DoCharEvents
            End If
            
            ' Convert Coord if needed
            If IsRound Then
            
                If CharData(i).WorldX < 0 Then
                    CharData(i).WorldX = TotalWidth + CharData(i).WorldX
                ElseIf CharData(i).WorldX > TotalWidth Then
                    CharData(i).WorldX = CharData(i).WorldX - TotalWidth
                End If
        
                If CharData(i).WorldY < 0 Then
                    CharData(i).WorldY = TotalHeight + CharData(i).WorldY
                ElseIf CharData(i).WorldY > TotalHeight Then
                    CharData(i).WorldY = CharData(i).WorldY - TotalHeight
                End If
                    
            End If
        
        End If
        
        ' Move the characters Light
        tmpEngine.MoveLight_Char CharData(i), OffsetX, OffsetY, TotalWidth, TotalHeight
        
    Next i
    
    
    For i = 0 To NumOfNPCs - 1
    
        ' Don't do if NPC is moving We assume NPCs are set up
        If NPCData(i).IsMoving Then
            
            ' Do its events
            NPCData(i).DoCharEvents
            
            ' Convert Coord if necassary
            If IsRound Then
            
                If NPCData(i).WorldX < 0 Then
                    NPCData(i).WorldX = TotalWidth + NPCData(i).WorldX
                ElseIf CharData(i).WorldX > TotalWidth Then
                    NPCData(i).WorldX = NPCData(i).WorldX - TotalWidth
                End If
        
                If NPCData(i).WorldY < 0 Then
                    NPCData(i).WorldY = TotalHeight + NPCData(i).WorldY
                ElseIf NPCData(i).WorldY > TotalHeight Then
                    NPCData(i).WorldY = NPCData(i).WorldY - TotalHeight
                End If
                    
            End If
                    
        ' Else if NPC Active then find if should move
        ElseIf NPCData(i).bActive Then
            Randomize
            Select Case Int((200 * Rnd) + 1)
                Case 1
                    MoveNPC i, MEUP
                Case 2
                    MoveNPC i, MEDOWN
                Case 3
                    MoveNPC i, MERIGHT
                Case 4
                    MoveNPC i, MELEFT
            End Select
            
        End If
        
        ' Move the NPCs light
        tmpEngine.MoveLight_NPC NPCData(i), OffsetX, OffsetY, TotalWidth, TotalHeight

    Next i
    
    ' Move the stationary Lights if needed
    For i = 0 To NumOfLights - 1
        tmpEngine.MoveLight_Map LightsDesc(i).TileX * TILEWIDTH, LightsDesc(i).TileY * TILEHEIGHT, LightsDesc(i).ID, OffsetX, OffsetY, TotalWidth, TotalHeight
    Next i
    
End Sub

' Check is character is set up ( Returns true or false)
Public Function IsCharacterSetUp(CharID As Integer) As Boolean
    IsCharacterSetUp = CharData(CharID).IsSet
End Function

' Get the tile's X offset from the tile surface ( Returns value)
Public Function GetTileOffsetX(i As Integer) As Long
    GetTileOffsetX = TileOffsetX(i)
End Function

' Get the tile's Y offset from the tile surface ( Returns value)
Public Function GetTileOffsetY(i As Integer) As Long
    GetTileOffsetY = TileOffsetY(i)
End Function

' Get the Middle tile ( Returns value)
Public Function GetTileMiddle(i As Integer, j As Integer) As Integer
    GetTileMiddle = TileArray(i, j).Middle
End Function

' Get the Top tile ( Returns value)
Public Function GetTileTop(i As Integer, j As Integer) As Integer
    GetTileTop = TileArray(i, j).Top
End Function

' Get the Bottom tile ( Returns value)
Public Function GetTileBottom(i As Integer, j As Integer) As Integer
    GetTileBottom = TileArray(i, j).Bottom
End Function

' Get the Data tile ( Returns value)
Public Function GetTileData(i As Integer, j As Integer) As Integer
    GetTileData = TileArray(i, j).Data
End Function

' Change Maps
Public Sub DoWarp(i As Integer, j As Integer)
    
    ' Save the NPCS and their position
    SaveNPCs
    
    AddScript "OldPosition " & sFilename & " " & CharData(ActiveChar).GetTileX & "," & CharData(ActiveChar).GetTileY
    
    ' Get the new map name
    sFilename = CommonPath & "\" & TileArray(i, j).EventString

    ' Load the map
    LoadMap
                
    ' Reset the characters (will only have one)
    ResetChar
    
    AddScript "ResetLights"
    
End Sub

' Set the tile bottom with given params
Public Sub SetTileMiddle(i As Integer, j As Integer, value As Integer)
    TileArray(i, j).Middle = value
End Sub

' Set the tile top with given params
Public Sub SetTileTop(i As Integer, j As Integer, value As Integer)
     TileArray(i, j).Top = value
End Sub

' Set the tile bottom with given params
Public Sub SetTileBottom(i As Integer, j As Integer, value As Integer)
    TileArray(i, j).Bottom = value
End Sub

' Set the tile Data with given params
Public Sub SetTileData(i As Integer, j As Integer, value As Integer, Optional sEventString As String = "")
    TileArray(i, j).Data = value
    TileArray(i, j).EventString = sEventString
End Sub

' GEt the event string of a tile
Public Function GetTileEventString(i As Integer, j As Integer) As String
    GetTileEventString = TileArray(i, j).EventString
End Function

' Get the Y-offset of the map on the screen
Public Property Get MapOffsetX() As Long
    MapOffsetX = OffsetX
End Property
 
' Get the Y-offset of the map on the screen
Public Property Get MapOffsetY() As Long
    MapOffsetY = OffsetY
End Property

' Create an empty map
Public Sub CreateMap()
Dim i As Integer, j As Integer
    
    ' Resize the array that holds the tile information
    ReDim TileArray(Width, Height)
    
    ' Make every tile's values 0
    For i = 0 To Width - 1
        For j = 0 To Height - 1
            TileArray(i, j).Bottom = 0
            TileArray(i, j).Top = 0
            TileArray(i, j).Middle = 0
            TileArray(i, j).Data = 0
            TileArray(i, j).EventString = ""
        Next j
    Next i

    ' Set it to have no NPCs
    ReDim NPCData(0)
    
End Sub

 ' Set the surface for the map
Public Sub SetTileSet(sName As String, iHeight As Integer, iWidth As Integer, tColorKey As Long)

    ' Set the name of the tile set
    SurfaceName = sName
    
    ' Set the tileset height
    TileSetHeight = iHeight
    
    ' Set the tileset width
    TileSetWidth = iWidth
    
    ' Set the color key
    TileColorKey = tColorKey
    
    ' Create the offsets for the tiles
    CreateTileSetOffset
    
End Sub

' Create the tile offsets from the tile set (Speeds it up by pre-calculating)
Private Sub CreateTileSetOffset()
Dim i As Integer, j As Integer
Dim tmpVal As Integer
  
    ' Set the size of the offset tables
    ReDim TileOffsetY(TileSetHeight * TileSetWidth)
    ReDim TileOffsetX(TileSetWidth * TileSetHeight)
    
    ' Start at 0
    tmpVal = 0
    
    ' Set up the tile offsets
    For i = 0 To TileSetHeight - 1
        For j = 0 To TileSetWidth - 1
            
            TileOffsetX(tmpVal) = j * TILEWIDTH
            TileOffsetY(tmpVal) = i * TILEHEIGHT
            
            ' Increment the count
            tmpVal = tmpVal + 1
            
        Next j
    Next i
End Sub

' Get the character with index i
Public Property Get Character_Data(i As Integer) As Character
    Set Character_Data = CharData(i)
End Property

' Gets an NPC
Public Property Get NPC_Data(i As Integer) As NPC_Character
    Set NPC_Data = NPCData(i)
End Property

' Sets an NPC
Public Sub SetNPC_Data(tmpNPC As NPC_Character, i As Integer)
    Set NPCData(i) = tmpNPC
End Sub

' save the map to a file
Public Function SaveMap() As Boolean
On Error Resume Next

    Dim lFile As Long
    Dim i As Integer, j As Integer
    Dim nLen As Byte
    Dim tmpString As String
    Dim NumOfNPCs As Byte

    ' Get the number of NPCS
    NumOfNPCs = NumberOfNPCs
        
    'Kill sFilename
    Kill sFilename
    
    ' Get destination address
    lFile = FreeFile
    
    ' opens the address for writinh
    Open sFilename For Binary As lFile
    
    On Error GoTo ErrorHandler
    
    ' Save Current map version (Save it to an integer because it is a const
    ' and may have a different size in the future, so want to keep same length)
    i = CurrentMapVersion
    Put #lFile, , i
    
    ' save the tile width of the map
    Put #lFile, , Width
    
    ' save the tile height of the map
    Put #lFile, , Height
    
    ' Save if the tile is round
    Put #lFile, , IsRound
    
    ' Save the name of the tile surface
    nLen = Len(SurfaceName)
    Put #lFile, , nLen
    Put #lFile, , SurfaceName
    
    ' Save the tile Surface's width and height
    Put #lFile, , TileSetWidth
    Put #lFile, , TileSetHeight
    
    ' Save the tile Surface's Colorkey
    Put #lFile, , TileColorKey
    
    ' Save the start position of characters that enter
    Put #lFile, , CharStartX
    Put #lFile, , CharStartY
    
    ' save the map data to the file
    For i = 0 To Width - 1
        For j = 0 To Height - 1
            ' Save the bitmap data
            Put #lFile, , TileArray(i, j).Bottom
            Put #lFile, , TileArray(i, j).Middle
            Put #lFile, , TileArray(i, j).Top
            
            ' Save the tile data
            Put #lFile, , TileArray(i, j).Data
            
            ' Save the string length
            nLen = Len(TileArray(i, j).EventString)
            Put #lFile, , nLen
            
            ' If equal to "" then don't even bother saving
            If nLen <> 0 Then
                Put #lFile, , TileArray(i, j).EventString
            End If
            
        Next j
    Next i
    
    ' Save number of NPCs
    Put #lFile, , NumOfNPCs
    
    ' Attach NPC's file name to the map
    For i = 0 To NumOfNPCs - 1
        
        ' Get the length of the filename
        tmpString = NPCData(i).sFilename
        nLen = Len(tmpString)
        
        ' Save the length and name
        Put #lFile, , nLen
        Put #lFile, , tmpString
    
    Next i
    
    ' Get the number of lights and save it
    nLen = NumberOfLights
    Put #lFile, , nLen
    
    ' Save their coords.  The Ids will be set at runtime
    For i = 0 To nLen - 1
        Put #lFile, , LightsDesc(i).TileX
        Put #lFile, , LightsDesc(i).TileY
    Next i
    
    ' Sav e all the NPCs states
    SaveNPCs
    
    ' closes the file
    Close lFile
    
    ' Return True since it was successful
    SaveMap = True
    Exit Function

ErrorHandler:
    
    ' Save failed, return false
    SaveMap = False

End Function

' Load the map from a file
Public Function LoadMap() As Boolean
    Dim lFile As Long
    Dim i As Integer, j As Integer
    Dim nLen As Byte
    Dim tmpString As String
    Dim NumOfNPCs As Byte

    lFile = FreeFile
    
    ' Open the file
    Open sFilename For Binary As lFile
    
    On Error GoTo ErrorHandler
    
    ' Get the version of the map and check it
    Get #lFile, , i
    If i <> CurrentMapVersion Then
        MsgBox "Wrong Map Version"
        Close lFile
        LoadMap = False
        Exit Function
    End If
    
    ' Load the tile width of the map
    Get #lFile, , Width
    
    ' Load the tile height of the map
    Get #lFile, , Height
    
    ' Load if the Map is round
    Get #lFile, , IsRound
    
    ' Resize the tile array
    ReDim TileArray(Width, Height)
    
    ' Load the length of the surface name
    Get #lFile, , nLen
    SurfaceName = Space$(nLen)
    
    ' Load the surface name
    Get #lFile, , SurfaceName
    
    ' Load the surface width and height
    Get #lFile, , TileSetWidth
    Get #lFile, , TileSetHeight
    
    ' Load the tile set color key
    Get #lFile, , TileColorKey
        
    ' Load character start positions
    Get #lFile, , CharStartX
    Get #lFile, , CharStartY
    
    ' Load the map data to the file
    For i = 0 To Width - 1
        For j = 0 To Height - 1
        
            ' Load the bitmap data
            Get #lFile, , TileArray(i, j).Bottom
            Get #lFile, , TileArray(i, j).Middle
            Get #lFile, , TileArray(i, j).Top
            
            ' Load the properties of the tile
            Get #lFile, , TileArray(i, j).Data
            
            ' GEt the length of its string
            Get #lFile, , nLen
            
            ' If zero, its not saved
            If nLen <> 0 Then
                TileArray(i, j).EventString = Space$(nLen)
                Get #lFile, , TileArray(i, j).EventString
            End If
        
        Next j
    Next i
    
    ' Get the number of NPCs
    Get #lFile, , NumOfNPCs
    
    ' Resize the NPC array
    ReDim NPCData(NumOfNPCs)
    
    ' Load all their filenames
    For i = 0 To NumOfNPCs - 1
    
        Get #lFile, , nLen
        tmpString = Space$(nLen)
    
        Get #lFile, , tmpString
    
        NPCData(i).sFilename = tmpString
    
    Next i
    
    ' Get the number of lights
    Get #lFile, , nLen
    
    ' Resize the array
    ReDim LightsDesc(nLen)
    
    ' Set their location
    For i = 0 To nLen - 1
        Get #lFile, , LightsDesc(i).TileX
        Get #lFile, , LightsDesc(i).TileY
    Next i
    
    ' closes the file
    Close lFile
    
    ' Load all the NPCs data
    LoadNPCs
    
    ' Create the offsets of the tiles
    CreateTileSetOffset
    
    ' Return True
    LoadMap = True
    IsSet = True
    
    Exit Function

ErrorHandler:
        
    ' Load failed so close file and return false
    Close lFile
    LoadMap = False

End Function

' Test the next move position and if can move the character (Returns if the char moved)
Public Function MoveCharacter(CharID As Integer, Direction As Integer) As Boolean

' Holds the temp data to determine if we can move to that spot
Dim NextX As Integer, NextY As Integer
Dim OldX As Integer, OldY As Integer
    
    ' If the character is moving, we shouldn't even check
    If CharData(CharID).IsMoving Then
        MoveCharacter = False
        Exit Function
    End If
    
    ' get the initial position
    NextX = CharData(CharID).GetTileX
    NextY = CharData(CharID).GetTileY

    ' Set the tile as not having a character
    TileArray(NextX, NextY).CharID = 0

    ' Set the initial position
    OldX = NextX
    OldY = NextY
    
    ' Get the next position
    Select Case Direction
        Case MERIGHT
            NextX = NextX + 1
        
        Case MELEFT
            NextX = NextX - 1
        
        Case MEUP
            NextY = NextY - 1
        
        Case MEDOWN
            NextY = NextY + 1
    
    End Select
    
    ' Set the face direction
    CharData(CharID).SetFaceDirection Direction
    
    ' Check if it is a round map
    If IsRound Then  ' Map is round

        ' If the next tile (X-Coord.) is less than zero make it the last tile in the map
        If NextX < 0 Then
            NextX = Width + NextX
          
        ' If it is past the width of the map make it the first tile
        ElseIf NextX >= Width Then
            NextX = NextX - Width
        End If
    
        ' Check the Y-Coord.
        If NextY < 0 Then
            NextY = Height + NextY
        ElseIf NextY >= Height Then
            NextY = NextY - Height
        End If
        
    Else ' Map not round, just stop charcter if needed
        If NextX = -1 Or NextX = Width Then Exit Function
        If NextY = -1 Or NextY = Height Then Exit Function
    End If
    
    ' If the spot we want to move to is open
    If TileArray(NextX, NextY).CharID = 0 And TileArray(NextX, NextY).Data = NotBlocked Then
        ' Can Move
        
        
        ' Set the CharID of the tile
        TileArray(NextX, NextY).CharID = CharID + 1
        
        ' Move the character in the specified direction
        CharData(CharID).MoveCharacter (Direction)
        
        ' Check tiles again and update character position
        If OldX <= 0 Or OldX >= Width - 1 Then
            CharData(CharID).SetTileX NextX
        End If
        
        If OldY <= 0 Or OldY >= Height - 1 Then
            CharData(CharID).SetTileY NextY
        End If
        
        ' The character Moved
        MoveCharacter = True
        
    Else ' Can't Move
        
        ' Set the CharID to the old tile
        TileArray(OldX, OldY).CharID = CharID + 1
        
        ' The character did not move
        MoveCharacter = False
    End If
        
End Function

' Test the next move position and if can move the character (Returns whether should hold input for while)
Public Function MoveActiveCharacter(Direction As Integer) As Boolean

' Holds the temp data to determine if we can move to that spot
Dim NextX As Integer, NextY As Integer
Dim OldX As Integer, OldY As Integer

    ' If the character is moving, we shouldn't even check
    If CharData(ActiveChar).IsMoving Then
        Exit Function
    End If
    
    If IsTalkBox Then
        Select Case Direction
            Case MERIGHT
                CursorIndex = CursorIndex + CursorOffset
            Case MELEFT
                CursorIndex = CursorIndex - CursorOffset
            Case MEUP
                CursorIndex = CursorIndex - 1
            Case MEDOWN
                CursorIndex = CursorIndex + 1
        End Select

        If CursorIndex < 0 Then CursorIndex = 0
        If CursorIndex > CursorMax Then CursorIndex = CursorMax
        
        UpdateTextBox
        
        MoveActiveCharacter = True
        
        Exit Function
    End If
    
    ' get the initial position
    NextX = CharData(ActiveChar).GetTileX
    NextY = CharData(ActiveChar).GetTileY

    ' Set the tile as not having a character
    TileArray(NextX, NextY).CharID = 0

    ' Set the initial position
    OldX = NextX
    OldY = NextY
    
    ' Get the next position
    Select Case Direction
        Case MERIGHT
            NextX = NextX + 1
        
        Case MELEFT
            NextX = NextX - 1
        
        Case MEUP
            NextY = NextY - 1
        
        Case MEDOWN
            NextY = NextY + 1
    
    End Select
    
    ' Set the face direction
    CharData(ActiveChar).SetFaceDirection Direction
    
    ' Check if it is a round map
    If IsRound Then  ' Map is round

        ' If the next tile (X-Coord.) is less than zero make it the last tile in the map
        If NextX < 0 Then
            NextX = Width + NextX
          
        ' If it is past the width of the map make it the first tile
        ElseIf NextX >= Width Then
            NextX = NextX - Width
        End If
    
        ' Check the Y-Coord.
        If NextY < 0 Then
            NextY = Height + NextY
        ElseIf NextY >= Height Then
            NextY = NextY - Height
        End If
        
    Else ' Map not round, just stop charcter if needed
        If NextX = -1 Or NextX = Width Then Exit Function
        If NextY = -1 Or NextY = Height Then Exit Function
    End If
    
    
    ' If a warp tile, do the warp
    If TileArray(NextX, NextY).Data = Warp Then
        DoWarp NextX, NextY
        MoveActiveCharacter = False
        Exit Function
    End If
    
    ' If the spot we want to move to is open
    If TileArray(NextX, NextY).CharID = 0 And TileArray(NextX, NextY).Data = NotBlocked Then
        ' Can Move
        
        
        ' Set the CharID of the tile
        TileArray(NextX, NextY).CharID = ActiveChar + 1
        
        ' Move the character in the specified direction
        CharData(ActiveChar).MoveCharacter (Direction)
        
        ' Check tiles again and update character position
        If OldX <= 0 Or OldX >= Width - 1 Then
            CharData(ActiveChar).SetTileX NextX
        End If
        
        If OldY <= 0 Or OldY >= Height - 1 Then
            CharData(ActiveChar).SetTileY NextY
        End If
        
    Else ' Can't Move
        
        ' Set the CharID to the old tile
        TileArray(OldX, OldY).CharID = ActiveChar + 1
        
    End If
    
    MoveActiveCharacter = False
        
End Function

' Adds a new character to the array (returns Characters ID)
Public Function AddCharacter(PlayerID As Long, Optional sFile As String = "") As Integer
Dim tmp As Integer
Dim NumOfChars As Integer

    NumOfChars = NumberOfCharacters
    ' See if there is an empty char
    tmp = FindEmptyChar
    
    If tmp = -1 Then ' No empty characters in memory
        
        ' Update the number of characters
        NumOfChars = NumOfChars + 1
    
        ' Resize the character array
        ReDim Preserve CharData(NumOfChars)
    
        ' Set the character id
        tmp = NumOfChars - 1
        
    End If

    ' If we got a start file use it
    If Not sFile = "" Then
        
        ' File givin, use it
        CharData(tmp).sFilename = sFile
        
        ' Load the character
        CharData(tmp).LoadCharacter
        
    Else ' No file
        
        ' Set as set-up
        CharData(tmp).IsSet = True
        
        CharData(tmp).InitTileSet "s02char.bmp", SpriteTileWidth, SpriteTileHeight, 0, 0
        
        ' Do the actual set-up
        CharData(tmp).SetUpChar PlayerID, "Temp", CharStartX, CharStartY
        
    End If
    
    ' Send the character id
    AddCharacter = tmp
    
    ' Set the CharID of the tile that they r placed on
    TileArray(CharData(tmp).GetTileX, CharData(tmp).GetTileY).CharID = tmp + 1
    
End Function

' Destroy the character
Public Sub DestroyChar(CharID As Integer)
    CharData(CharID).Reset
End Sub

' Destroy the character by using player id
Public Sub DestroyPlayer(lPlayerId As Long)
    Dim i As Integer
    Dim NumOfChars As Integer
    
    NumOfChars = NumberOfCharacters
    
    ' Cycle through all the characters until the one we want is found
    Do While i < NumOfChars
        If CharData(i).ID = lPlayerId Then
            CharData(i).Reset
        End If
        
        i = i + 1
    Loop

End Sub

' Finds the next empty character (Returns ID or -1 if not found)
Private Function FindEmptyChar() As Integer
Dim i As Integer

    Dim NumOfChars As Integer
    NumOfChars = NumberOfCharacters

    ' Loops until it finds an empty character
    Do While i < NumOfChars
        
        ' Look to see if it is set
        If CharData(i).IsSet = False Then
            
            ' If not then use it
            FindEmptyChar = i
            Exit Function
            
        End If
        i = i + 1
    Loop

    FindEmptyChar = -1
    
End Function

' Returns the characters name
Public Function GetCharName(CharID As Integer) As String
    GetCharName = CharData(CharID).CharacterName
End Function

' Returns the characters name
Public Function GetNPCName(NPCId As Integer) As String
    GetNPCName = NPCData(NPCId).CharacterName
End Function

' Returns the X-Position (tile number) of the character
Public Function GetCharTileX(CharID As Integer) As Integer
    GetCharTileX = CharData(CharID).GetTileX
End Function

' Returns the Y-Position (tile number) of the character
Public Function GetCharTileY(CharID As Integer) As Integer
    GetCharTileY = CharData(CharID).GetTileY
End Function

' Returns the Name of the tile set
Public Function GetCharTileName(CharID As Integer) As String
    GetCharTileName = CharData(CharID).TileName
End Function

' Returns the X-Position (tile number) of the character
Public Function GetNPCTileX(CharID As Integer) As Integer
    GetNPCTileX = NPCData(CharID).GetTileX
End Function

' Returns the Y-Position (tile number) of the character
Public Function GetNPCTileY(CharID As Integer) As Integer
    GetNPCTileY = NPCData(CharID).GetTileY
End Function

' Returns the Name of the tile set
Public Function GetNPCTileName(CharID As Integer) As String
    GetNPCTileName = NPCData(CharID).TileName
End Function

' Set the left corner of visible map lAmount pixels down
Public Function ScrollRight(Amount As Integer) As Boolean
    
    OffsetX = OffsetX + Amount
    
    If Not IsRound Then
        If (Width * TILEWIDTH) - OffsetX < RESX Then
        
            ' no more scrolling possible, border reached
            If (TILEWIDTH * Width) < RESX Then
                OffsetX = 0
            Else
                OffsetX = (Width * TILEWIDTH) - RESX
            End If
            ScrollRight = False
        Else
        
            ' border not reached
            ScrollRight = True
        End If
    Else
        If Abs(OffsetX) >= Width * TILEWIDTH Then OffsetX = 0
        ScrollRight = True
    End If
    
End Function

' Set the left corner of visible map lAmount pixels down
Public Function ScrollLeft(Amount As Integer) As Boolean
    OffsetX = OffsetX - Amount
    
    If Not IsRound Then
        If OffsetX < 0 Then
        
            ' no more scrolling possible, border reached
            OffsetX = 0
            ScrollLeft = False
        Else
            ' border not reached
            ScrollLeft = True
        End If
    Else
        If OffsetX <= 0 Then
            OffsetX = Width * TILEWIDTH + OffsetX
        End If
        ScrollLeft = True
    End If
End Function

' Set the top corner of visible map lAmount pixels down
Public Function ScrollDown(Amount As Integer) As Boolean
    OffsetY = OffsetY + Amount
    
    If Not IsRound Then
        If (Height * TILEHEIGHT) - OffsetY < RESY Then
        
            ' no more scrolling possible, border reached
            OffsetY = (Height * TILEHEIGHT) - RESY
            ScrollDown = False
        Else
        
            ' border not reached
            ScrollDown = True
        End If
    Else
        If Abs(OffsetY) >= Height * TILEHEIGHT Then OffsetY = 0
        ScrollDown = True
    End If
End Function

' Set the top corner of visible map lAmount pixels up
Public Function ScrollUp(Amount As Integer) As Boolean
    OffsetY = OffsetY - Amount
    
    If Not IsRound Then
        If OffsetY < 0 Then
        
            ' no more scrolling possible, border reached
            OffsetY = 0
            ScrollUp = False
        Else
        
            ' border not reached
            ScrollUp = True
        End If
    Else
        If OffsetY <= 0 Then
            OffsetY = Height * TILEHEIGHT + OffsetY
        End If
        ScrollUp = True
    End If
End Function

' Check to see if there is something infront of the character
Public Function CheckData() As Boolean
Dim tmpID As Integer
Dim tmpBool As Boolean
Dim tmpString As String
Dim Direction As Integer
Dim TalkerID As Integer
Dim i As Integer
Dim j As Integer

    If IsTalkBox Then
        EndTalk
        CheckData = True
        Exit Function
    End If
    
    If CharData(ActiveChar).IsMoving Then
        CheckData = False
        Exit Function
    End If

    ' Get the characters tile position
    i = CharData(ActiveChar).GetTileX
    j = CharData(ActiveChar).GetTileY
    
    ' get their face direction
    Direction = CharData(ActiveChar).GetFaceDirection
    
    ' Find which tile is infornt of the character
    Select Case Direction
        Case MEUP
            j = j - 1
        Case MEDOWN
            j = j + 1
        Case MERIGHT
            i = i + 1
        Case MELEFT
            i = i - 1
    End Select
    
    
    ' Check if it is a round map
    If IsRound Then  ' Map is round

        ' If the next tile (X-Coord.) is less than zero make it the last tile in the map
        If i < 0 Then
            i = Width + i
          
          ' If it is past the width of the map make it the first tile
        ElseIf i >= Width Then
            i = i - Width
        End If
    
        ' Check the Y-Coord.
        If j < 0 Then
            j = Height + j
        ElseIf j >= Height Then
            j = j - Height
        End If
        
    Else ' Map not round, just stop charcter if needed
        If i = -1 Or i = Width Then Exit Function
        If j = -1 Or j = Height Then Exit Function
    End If
    
    ' Get the ID of the Character/NPC in the next tile
    tmpID = TileArray(i, j).CharID
    TalkerID = -1
    
    If tmpID >= 2000 Then ' Its an NPC
        
        ' Get their talk string
        tmpString = NPCData(tmpID - 2000).GetTalkString
            
            If Not tmpString = "" Then
                ' If not nothing, then display box
                NPCData(tmpID - 2000).bActive = False
                TalkerID = tmpID - 2000
            Else
                ' If nothing do nothing
                TalkerID = -1
            End If
    ElseIf TileArray(i, j).Data > 1 Then ' If its an event, decode it
        tmpBool = DecodeTileEvent(i, j, Direction)
    End If
    
    
    If Not TalkerID = -1 Then
        IsTalkBox = True
        
        ' Send the string for the talkbox
        AddScript ("TalkBox " & TalkerID & " " & tmpString)
        
        CheckData = True
    ElseIf tmpBool Then
        IsTalkBox = True
        CheckData = True
    Else
        CheckData = False
    End If
    
    
End Function

Public Sub AddFormattedBox(sString As String)

If IsTalkBox Then Exit Sub

AddScript ("FormattedBox " & sString)

End Sub

Public Sub AddInputBox(sString As String)
Dim tmpString As String
Dim splitString() As String
If IsTalkBox Then Exit Sub

    splitString = Split(sString, "<INPUTBOX ")
    splitString = Split(splitString(1), ">")
    splitString = Split(splitString(0), " ")
    CursorMax = UBound(splitString())

    splitString = Split(sString, "<CURSOROFFSET ")
    
    If UBound(splitString()) = 0 Then
        CursorOffset = 1
    Else
        splitString = Split(splitString(1), ">")
        CursorOffset = Val(splitString(0))
    End If

    AddScript ("InputBox " & sString)

End Sub

' Returns ID of talkstring
Public Function FindTalkString(Optional OnlyInputBox As Boolean = False) As Integer
Dim i As Integer
Dim NumofEvents As Integer
Dim tmpString() As String

    NumofEvents = UBound(ActiveScripts())
    
    If OnlyInputBox Then
        For i = 0 To NumofEvents - 1
            tmpString() = Split(ActiveScripts(i), " ")
        
            If tmpString(0) = "InputBox" Then
                FindTalkString = i
                Exit Function
            End If
        
        Next i
    Else
        For i = 0 To NumofEvents - 1
            tmpString() = Split(ActiveScripts(i), " ")
        
            If tmpString(0) = "InputBox" Or tmpString(0) = "TalkBox" Or tmpString(0) = "TextBox" Or tmpString(0) = "FormattedBox" Then
                FindTalkString = i
                Exit Function
            End If
        
        Next i
    End If

    
    FindTalkString = -1
    
End Function

Public Sub EndTalk()
Dim i As Integer
Dim tmpString() As String

    i = FindTalkString()
    
    If i = -1 Then Exit Sub
    
    tmpString() = Split(ActiveScripts(i), " ")
    
    If tmpString(0) = "TalkBox" Then
        NPCData(Val(tmpString(1))).bActive = True
    End If
    
    DeleteScript i
    
    IsTalkBox = False
    
End Sub

Public Function DecodeActiveEvents(tmpEngine As D3D8Engine) As Boolean
Dim i As Integer, iSize As Integer
Dim tmpString() As String
Dim tmpPos() As String
Dim PosX As Long, PosY As Long
Dim DestX As Long, DestY As Long
Dim tmpTile As Integer
Dim OffX As Long, OffY As Long
Dim lHeight As Long, lWidth As Long
Dim DeleteInts() As Integer
Dim NumOfDeletes As Integer
Dim tmpTalkString As String
    
    iSize = UBound(ActiveScripts())

    lHeight = TileSetHeight * TILEHEIGHT
    lWidth = TileSetWidth * TILEWIDTH
    
    NumOfDeletes = 0
    ReDim Preserve DeleteInts(NumOfDeletes)
    
    
    For i = 0 To iSize - 1
    
        tmpString() = Split(ActiveScripts(i), " ")
    
        Select Case tmpString(0)
            Case "FormattedBox"
                IsTalkBox = True
                tmpEngine.DrawFormattedBox Mid(ActiveScripts(i), 13)
            Case "InputBox"
                IsTalkBox = True
                tmpEngine.DrawFormattedBox Mid(ActiveScripts(i), 10) & " <CURSOR " & CursorIndex & ">"
            Case "TalkBox"
                
                tmpTalkString = Mid(ActiveScripts(i), Len(tmpString(0) & " " & tmpString(1) & " ") + 1)
                
                tmpEngine.DrawTextBox tmpTalkString, COLOR_BLACK
            Case "TextBox"
                
                tmpTalkString = Mid(ActiveScripts(i), Len(tmpString(0)) + 2)
                
                tmpEngine.DrawTextBox tmpTalkString, COLOR_BLACK
                
            Case "ResetLights"
                ResetLightIDs tmpEngine, 0
                
                NumOfDeletes = NumOfDeletes + 1
                ReDim Preserve DeleteInts(NumOfDeletes)
                    
                DeleteInts(NumOfDeletes - 1) = i
                
            Case "MoveTile"
                Dim MapWidth As Long
                Dim MapHeight As Long
                
                MapWidth = Width * TILEWIDTH
                MapHeight = Height * TILEHEIGHT
                
                tmpTile = Val(tmpString(1))
                OffX = TileOffsetX(tmpTile)
                OffY = TileOffsetY(tmpTile)
            
                tmpPos() = Split(tmpString(2), ",")
                PosX = Val(tmpPos(0))
                PosY = Val(tmpPos(1))
            
                tmpPos() = Split(tmpString(3), ",")
                DestX = Val(tmpPos(0))
                DestY = Val(tmpPos(1))
                
                If PosX = DestX And PosY = DestY Then
                    NumOfDeletes = NumOfDeletes + 1
                    ReDim Preserve DeleteInts(NumOfDeletes)
                    
                    DeleteInts(NumOfDeletes - 1) = i
                    
                    TileArray(PosX / TILEWIDTH, PosY / TILEHEIGHT).Middle = tmpTile
                    TileArray(PosX / TILEWIDTH, PosY / TILEHEIGHT).Data = Moveable
                Else
                
                    If PosX < DestX Then
                        PosX = PosX + PIXELSPEED
                    ElseIf PosX > DestX Then
                        PosX = PosX - PIXELSPEED
                    End If
                    
                    If PosY < DestY Then
                        PosY = PosY + PIXELSPEED
                    ElseIf PosY > DestY Then
                        PosY = PosY - PIXELSPEED
                    End If
                    
                    ActiveScripts(i) = "MoveTile " & tmpTile & " " & PosX & "," & PosY & " " & DestX & "," & DestY

                End If

                ' Convert to current view location
                If OffsetX > MapWidth - RESX Then
                    If PosX > OffsetX Then
                        PosX = PosX - OffsetX
                    Else
                        PosX = PosX + (MapWidth - OffsetX)
                    End If
                Else
                    PosX = PosX - OffsetX
                End If

                If OffsetY > MapHeight - RESY Then
                    If PosY > OffsetY Then
                        PosY = PosY - OffsetY
                    Else
                        PosY = PosY + (MapHeight - OffsetY)
                    End If
                Else
                    PosY = PosY - OffsetY
                End If

                tmpEngine.DrawSprite SurfaceName, PosX, PosY, OffX, OffY, lWidth, lHeight

        End Select
    Next i
            
    For i = 0 To NumOfDeletes - 1
        DeleteScript DeleteInts(i)
    Next i
            
End Function

' Decodes and executes a Tile Event
Private Function DecodeTileEvent(TileX As Integer, TileY As Integer, Direction As Integer) As Boolean
Dim tmpString() As String
Dim sString As String

    ' Select the event
    Select Case TileArray(TileX, TileY).Data
        
        Case SignString ' Its a sign
            ' Return the string to display
            DecodeTileEvent = True
            AddScript ("TextBox " & TileArray(TileX, TileY).EventString)

        
        Case TreasureChest ' Its a treasure chest
            
            ' Get the string of what to tell the character and ther item
            tmpString() = Split(TileArray(TileX, TileY).EventString, ";")
            
            ' First comes the string, then the data
            DecodeTileEvent = True
            AddScript ("TextBox " & tmpString(0))

            ' Update the tile sprite and make it blocked
            TileArray(TileX, TileY).Middle = TileArray(TileX, TileY).Middle + 1
            TileArray(TileX, TileY).Data = Blocked
            
            ' Use tmpstring(1) to tell what item(s) have been found
            
        Case Moveable ' Its a moveable item
            
            ' Finde the direction the character is facing
            Select Case Direction
                ' Only move it if next space isn't blocked by anything
                
                Case MEUP
                    If TileArray(TileX, TileY - 1).Data = NotBlocked And TileArray(TileX, TileY - 1).CharID = 0 Then
                        TileArray(TileX, TileY - 1).Data = Blocked
                        AddScript ("MoveTile " & TileArray(TileX, TileY).Middle & " " & TileX * TILEWIDTH & "," & TileY * TILEHEIGHT & " " & TileX * TILEWIDTH & "," & (TileY - 1) * TILEHEIGHT)
                        TileArray(TileX, TileY).Middle = 0
                        TileArray(TileX, TileY).Data = NotBlocked
                    End If
                
                Case MEDOWN
                    If TileArray(TileX, TileY + 1).Data = NotBlocked And TileArray(TileX, TileY + 1).CharID = 0 Then
                        TileArray(TileX, TileY + 1).Data = Blocked
                        AddScript ("MoveTile " & TileArray(TileX, TileY).Middle & " " & TileX * TILEWIDTH & "," & TileY * TILEHEIGHT & " " & TileX * TILEWIDTH & "," & (TileY + 1) * TILEHEIGHT)
                        TileArray(TileX, TileY).Middle = 0
                        TileArray(TileX, TileY).Data = NotBlocked
                    End If
                
                Case MELEFT
                    If TileArray(TileX - 1, TileY).Data = NotBlocked And TileArray(TileX - 1, TileY).CharID = 0 Then
                        TileArray(TileX - 1, TileY).Data = Blocked
                        AddScript ("MoveTile " & TileArray(TileX, TileY).Middle & " " & TileX * TILEWIDTH & "," & TileY * TILEHEIGHT & " " & (TileX - 1) * TILEWIDTH & "," & TileY * TILEHEIGHT)
                        TileArray(TileX, TileY).Middle = 0
                        TileArray(TileX, TileY).Data = NotBlocked
                    End If
                    
                Case MERIGHT
                    If TileArray(TileX + 1, TileY).Data = NotBlocked And TileArray(TileX + 1, TileY).CharID = 0 Then
                        TileArray(TileX + 1, TileY).Data = Blocked
                        AddScript ("MoveTile " & TileArray(TileX, TileY).Middle & " " & TileX * TILEWIDTH & "," & TileY * TILEHEIGHT & " " & (TileX + 1) * TILEWIDTH & "," & TileY * TILEHEIGHT)
                        TileArray(TileX, TileY).Middle = 0
                        TileArray(TileX, TileY).Data = NotBlocked
                    End If
            End Select
                        
        ' Not implemented yet
        Case Treasure
            ' Get the string of what to tell the character and ther item
            tmpString() = Split(TileArray(TileX, TileY).EventString, ";")
            
            ' First comes the string, then the data
            DecodeTileEvent = True
            AddScript ("TextBox " & tmpString(0))
            
            ' Update the tile sprite and make it blocked
            TileArray(TileX, TileY).Middle = 0
            TileArray(TileX, TileY).Data = Blocked
            
        Case HiddenTreasure
            ' Get the string of what to tell the character and ther item
            tmpString() = Split(TileArray(TileX, TileY).EventString, ";")
            
            ' First comes the string, then the data
            DecodeTileEvent = True
            AddScript ("TextBox " & tmpString(0))
            
            ' Update the tile sprite and make it blocked
            TileArray(TileX, TileY).Middle = TileArray(TileX, TileY).Middle + 1
            TileArray(TileX, TileY).Data = Blocked
            
        Case CustomString
            AddScript TileArray(TileX, TileY).EventString
            
    End Select



End Function

' Check to see if the screen should be scrolled
Public Function CheckScroll(Direction As Integer) As Boolean

Dim lPosX As Long
Dim lPosY As Long

Dim tBool As Boolean
    
    tBool = False

    ' Get the characters current position
    lPosX = CharData(0).PosX
    lPosY = CharData(0).PosY

    Select Case Direction
        Case MELEFT
            ' Is the char 1/2 way across the screen
            If lPosX - PIXELSPEED < RESX * EdgePercent Then
                tBool = ScrollLeft(PIXELSPEED)
            Else
            
                If Not lPosX > RESX * EdgePercent Then
                    ' just move map
                    tBool = ScrollLeft(PIXELSPEED)
                End If
            End If
            
        Case MERIGHT
            ' is the character less the 320 pixels near to the map border?
            If lPosX + PIXELSPEED > RESX * (1 - EdgePercent) Then
                tBool = ScrollRight(PIXELSPEED)
            Else
            ' is the character in the left half of the screen?
                If Not lPosX < RESX * (1 - EdgePercent) Then
                    tBool = ScrollRight(PIXELSPEED)
                End If
            End If
        Case MEUP
            ' is the character less the 240 pixels near to the map border?
            If lPosY - PIXELSPEED < RESY * EdgePercent Then
                tBool = ScrollUp(PIXELSPEED)
            Else
                ' is the character in the lower half of the screen?
                If Not lPosY > RESY * EdgePercent Then
                    tBool = ScrollUp(PIXELSPEED)
                End If
            End If
        Case MEDOWN
            ' is the character less the 240 pixels near to the map border?
            If lPosY + PIXELSPEED > RESY * (1 - EdgePercent) Then
                tBool = ScrollDown(PIXELSPEED)
            Else
                ' is the character in the upper half of the screen?
                If Not lPosY < RESY * (1 - EdgePercent) Then
                    tBool = ScrollDown(PIXELSPEED)
                End If
            End If
    End Select
    
    CheckScroll = tBool
    
End Function


' Set intiale variables
Private Sub Class_Initialize()
    
    ' Set the normal view settings
    SpecialVisible = False
    BottomVisible = True
    MiddleVisible = True
    TopVisible = True
    
    ' Size the arrays
    ReDim LightsDesc(0) As LightPos
    ReDim CharData(0) As New Character
    ReDim NPCData(0) As New NPC_Character
    ReDim ActiveScripts(1)
    
    ActiveScripts(0) = "ResetLights"
    
End Sub

' Set the character that moves the screen
Public Property Let ActiveCharacter(CharID As Integer)
    ActiveChar = CharID
End Property

Public Property Get GetActiveCharacter()
    GetActiveCharacter = ActiveChar
End Property

' Adds a new character to the array (returns Characters ID)
Public Sub AddNPC(Optional sFile As String = "")
Dim tmp As Integer
Dim NumOfNPCs As Integer

    NumOfNPCs = NumberOfNPCs
        
    tmp = NumOfNPCs
    
    ' Update the number of characters
    NumOfNPCs = NumOfNPCs + 1
    
    ' Resize the character array
    ReDim Preserve NPCData(NumOfNPCs)
     
    ' Set as set-up
    NPCData(tmp).InitTileSet "s02char.bmp", SpriteTileWidth, SpriteTileHeight, 4, 0
        
    ' Do the actual set-up
    NPCData(tmp).SetUpChar "NPC1", 3, 3
        
    NPCData(tmp).sFilename = "Temp.npc"
    
    TileArray(NPCData(tmp).GetTileX, NPCData(tmp).GetTileY).CharID = tmp + 2000
    
End Sub

Public Function FindCharWarpPos() As Integer
Dim i As Integer
Dim NumofEvents As Integer
Dim tmpString() As String

    NumofEvents = UBound(ActiveScripts())
    
    
    For i = 0 To NumofEvents - 1
        tmpString() = Split(ActiveScripts(i), " ")
        
        If tmpString(0) = "OldPosition" And LCase(tmpString(1)) = LCase(sFilename) Then
            FindCharWarpPos = i
            Exit Function
        End If
        
    Next i

    FindCharWarpPos = -1
    
End Function

' Reset the chars from a map change.  If exiting a world map, save the location
Public Sub ResetChar()
Dim tmpChar As Character
Dim tmpString() As String
Dim tmpPos() As String
Dim TileX As Integer
Dim TileY As Integer
Dim AScript As Integer

    Set tmpChar = CharData(ActiveChar)
    
    ReDim Preserve CharData(1)
    
    AScript = FindCharWarpPos

    ' If exiting a world map, save the location
    If AScript = -1 Then

        tmpChar.SetTileX CharStartX
        tmpChar.SetTileY CharStartY
        
    Else
        ' Load the location
        
        tmpString() = Split(ActiveScripts(AScript), " ")
        tmpPos() = Split(tmpString(2), ",")
        
        tmpChar.SetTileX Val(tmpPos(0))
        tmpChar.SetTileY Val(tmpPos(1))
        
        DeleteScript AScript
        
    End If
    
    ' Get offsets of the screen, so character is as close to the center as possible
    If tmpChar.GetTileX * TILEWIDTH > Width * TILEWIDTH - RESX Then
        OffsetX = Width * TILEWIDTH - RESX
    Else
        OffsetX = tmpChar.GetTileX * TILEWIDTH - RESX / 2
    End If
    
    If tmpChar.GetTileY * TILEHEIGHT > Height * TILEHEIGHT - RESY Then
        OffsetY = Height * TILEHEIGHT - RESY
    Else
        OffsetY = tmpChar.GetTileY * TILEHEIGHT - RESY / 2
    End If
    
    ' Set their world position
    tmpChar.WorldX = tmpChar.GetTileX * TILEWIDTH
    tmpChar.WorldY = tmpChar.GetTileY * TILEHEIGHT
        
    ' Set them as standing
    tmpChar.SetState 0
    
    ' Check offsets
    If Not IsRound Then
        If OffsetX < 0 Then OffsetX = 0
        If OffsetY < 0 Then OffsetY = 0
    End If
    
    ' Update Position if needed
    If OffsetX = 0 And OffsetY = 0 Then
        tmpChar.PosX = tmpChar.WorldX
        tmpChar.PosY = tmpChar.WorldY
    Else
    
        tmpChar.PosX = tmpChar.WorldX - OffsetX
        tmpChar.PosY = tmpChar.WorldY - OffsetY
        
    End If
    
    ' SEt the char up
    Set CharData(0) = tmpChar

End Sub

' Test the next move position and if can move the character (Returns if the character moved)
Public Function MoveNPC(CharID As Integer, Direction As Integer) As Boolean

' Holds the temp data to determine if we can move to that spot
Dim NextX As Integer, NextY As Integer
Dim OldX As Integer, OldY As Integer
    
    ' If the character is moving, we shouldn't even check
    If NPCData(CharID).IsMoving Then
        MoveNPC = False
        Exit Function
    End If
    
    ' get the initial position
    NextX = NPCData(CharID).GetTileX
    NextY = NPCData(CharID).GetTileY

    ' Reset the tiles CharID
    TileArray(NextX, NextY).CharID = 0

    ' Set the initial position
    OldX = NextX
    OldY = NextY
    
    ' Get the next position
    Select Case Direction
        Case MERIGHT
            NextX = NextX + 1
        
        Case MELEFT
            NextX = NextX - 1
        
        Case MEUP
            NextY = NextY - 1
        
        Case MEDOWN
            NextY = NextY + 1
    
    End Select
    
    ' Set the face direction
    NPCData(CharID).SetFaceDirection Direction
    
    ' Check if it is a round map
    If IsRound Then  ' Map is round

        ' If the next tile (X-Coord.) is less than zero make it the last tile in the map
        If NextX < 0 Then
            NextX = Width + NextX
          
          ' If it is past the width of the map make it the first tile
        ElseIf NextX >= Width Then
            NextX = NextX - Width
        End If
    
        ' Check the Y-Coord.
        If NextY < 0 Then
            NextY = Height + NextY
        ElseIf NextY >= Height Then
            NextY = NextY - Height
        End If
        
        ' Update the characters position  (****Need to update World too, not implimented yet****)
        If NPCData(CharID).WorldX < 0 Then
            NPCData(CharID).WorldX = Width * TILEWIDTH - NPCData(CharID).WorldX
        ElseIf NPCData(CharID).WorldX > Width * TILEWIDTH Then
            NPCData(CharID).WorldX = NPCData(CharID).WorldX - Width * TILEHEIGHT
        End If
        
        If NPCData(CharID).WorldY < 0 Then
            NPCData(CharID).WorldY = Height * TILEHEIGHT - NPCData(CharID).WorldY
        ElseIf NPCData(CharID).WorldY > Height * TILEHEIGHT Then
            NPCData(CharID).WorldY = NPCData(CharID).WorldY - Height * TILEHEIGHT
        End If
        
    Else ' Map not round, just stop charcter if needed
        If NextX = -1 Or NextX = Width Then Exit Function
        If NextY = -1 Or NextY = Height Then Exit Function
    End If
    
    
    ' If the spot we want to move to is open
    If TileArray(NextX, NextY).CharID = 0 And TileArray(NextX, NextY).Data = NotBlocked Then
    
        ' Set the new tiles CharID
        TileArray(NextX, NextY).CharID = CharID + 2000
        
        ' Move the character in the specified direction
        NPCData(CharID).MoveCharacter (Direction)
        
        ' Check tiles again and update character position
        If OldX <= 0 Or OldX >= Width - 1 Then
            NPCData(CharID).SetTileX NextX
        End If
        
        If OldY <= 0 Or OldY >= Height - 1 Then
            NPCData(CharID).SetTileY NextY
        End If
        
        ' Return that the character moved
        MoveNPC = True
    Else
        ' Set the Old Tiles CharID
        TileArray(OldX, OldY).CharID = CharID + 2000
        
        ' The character did not move
        MoveNPC = False
    End If
    
End Function

' Re-assign all light Ids for map change
Public Sub ResetLightIDs(tmpEngine As D3D8Engine, StartID As Integer)
Dim i As Integer
Dim CurrentLightID As Integer
Dim NumOfChars As Integer
Dim NumOfNPCs As Integer

    ' Get number of NPCS and Chars
    NumOfNPCs = NumberOfNPCs
    NumOfChars = NumberOfCharacters
    
    ' turn off lights
    tmpEngine.TurnOffLights

    ' Start  at the first availible
    CurrentLightID = StartID

    ' Set the character lights
    For i = 0 To NumOfChars - 1
        
        ' Set it up
        tmpEngine.SetupLights CurrentLightID, D3DLIGHT_POINT
        
        ' Assign the light to the character
        CharData(i).LightID = CurrentLightID
        
        ' Increment counter
        CurrentLightID = CurrentLightID + 1
        
    Next i
    
    ' SEt the NPCs lights
    For i = 0 To NumOfNPCs - 1
        
        ' Set it up
        tmpEngine.SetupLights CurrentLightID, D3DLIGHT_POINT

        ' Assign the light to the NPC
        NPCData(i).LightID = CurrentLightID
        
        ' Increment the counter
        CurrentLightID = CurrentLightID + 1
    Next i
    
    ' Set yp the maps stationary lights
    For i = 0 To UBound(LightsDesc()) - 1
        
        ' Set it up
        tmpEngine.SetupLights CurrentLightID
        
        ' Assign it
        LightsDesc(i).ID = CurrentLightID
    
        ' Increment counter
        CurrentLightID = CurrentLightID + 1
            
    Next i
    
End Sub

' Save all NPCs included in this map
Public Sub SaveNPCs()
Dim i As Integer
Dim NumOfNPCs As Integer

    NumOfNPCs = NumberOfNPCs
    
    For i = 0 To NumOfNPCs - 1
        
        ' Save the NPC
        NPCData(i).SaveCharacter
            
    Next i
    
End Sub

' Load all NPCs included in this map
Public Sub LoadNPCs()
Dim i As Integer
Dim NumOfNPCs As Integer

    NumOfNPCs = NumberOfNPCs
    
    For i = 0 To NumOfNPCs - 1
        
        ' Load the NPC
        NPCData(i).LoadCharacter
        
        ' Set the tile's CharID
        TileArray(NPCData(i).GetTileX, NPCData(i).GetTileY).CharID = i + 2000
    Next i
End Sub

Private Sub UpdateTextBox()
Dim Index As Integer
Dim tmpString() As String
Dim LenString() As String
Dim RightHalf As String
Dim tmpLength As Long

    Index = FindTalkString(True)
    
    If Index = -1 Then Exit Sub
    
    tmpString = Split(ActiveScripts(Index), "<CURSOR ")
    
    DeleteScript (Index)

    AddScript tmpString(0) & "<CURSOR " & CursorIndex & ">"
End Sub
