Attribute VB_Name = "ModConst"
Option Explicit

' Used for Frame Counter
Declare Function GetTickCount Lib "kernel32" () As Long
Declare Function timeGetTime Lib "winmm.dll" () As Long

' The map verison we now are using
Global Const CurrentMapVersion = 2

' Display Variables
Public IsFullscreen As Boolean
Public RESX As Integer
Public RESY As Integer
Public BPP As Integer
Public CommonPath As String

' Will be changed for other resolutions , but for now 640X480
Public changeX As Double          ' 10 / (ResX/TileWidth)
Public changeY As Double       ' 10 / (ResY/TileHeight)

' Tile Settings
Public TILEHEIGHT As Integer
Public TILEWIDTH As Integer
Public ColorKeyTile As Long

' How big is the character bitmap
Global Const SpriteTileHeight = 8
Global Const SpriteTileWidth = 8

' Color Constants
Global Const COLOR_BLACK = &HFF000000
Global Const COLOR_RED = &HFFFF0000
Global Const COLOR_MAGENTA = &HFFFF00FF
Global Const COLOR_WHITE = &HFFFFFFFF

' Color of the fog
Global Const FOG_COLOR = &H4466

' The classes of the program
Public oMap As New clsMap

' The percent of how close the charcater can get to the edge before scrolling
Public EdgePercent As Double

'Flexible vertex format the describes untransformed and unlit vertices.
Global Const FVF = D3DFVF_XYZ Or D3DFVF_NORMAL Or D3DFVF_TEX1

' Max number of particles
Global Const nParticles = 300

'This structure describes a transformed and lit vertex.
Public Type TLVERTEX
    x As Single
    y As Single
    Z As Single
    nx As Single
    ny As Single
    nz As Single
    tu As Single
    tv As Single
End Type

Public Type PARTICLE
    x As Single     'World Space Coordinates
    y As Single
    Z As Single
    vX As Single    'Speed and Direction
    vY As Single
    vZ As Single
    Status As Boolean 'Does he even exist?
End Type

Public Type LightPos
    TileX As Integer
    TileY As Integer
    ID As Integer
End Type

' Predefined events
Enum Events
    NotBlocked = 0
    Blocked = 1
    Moveable = 2
    Treasure = 3
    HiddenTreasure = 4
    SignString = 5
    Warp = 6
    TreasureChest = 7
    CustomString = 20
End Enum

' Direct Input Constants
Global Const BufferSize = 10

' Status constants
Global Const MERIGHT = 2
Global Const MELEFT = 4
Global Const MEUP = 1
Global Const MEDOWN = 3
Global Const METALK = 5

' Constants for the movement of the sprite and map
Global Const PIXELSPEED = 2                            ' How many pixels we move in one turn
Global Const FRAMEWAIT = 10                            ' How many frames should we wait inbetween moves
Global Const MOVEFRAMES = 2                            ' How many different frames for each direction does the character have

Public Type Tile
    Bottom As Integer
    Middle As Integer
    Top As Integer
    Data As Byte
    CharID As Integer
    EventString As String
End Type

' DirectInput keycode constants
Global Const DIK_ESCAPE = 1
Global Const DIK_1 = 2
Global Const DIK_2 = 3
Global Const DIK_3 = 4
Global Const DIK_4 = 5
Global Const DIK_5 = 6
Global Const DIK_6 = 7
Global Const DIK_7 = 8
Global Const DIK_8 = 9
Global Const DIK_9 = 10
Global Const DIK_0 = 11
Global Const DIK_MINUS = 12
Global Const DIK_EQUALS = 13
Global Const DIK_BACKSPACE = 14
Global Const DIK_TAB = 15
Global Const DIK_Q = 16
Global Const DIK_W = 17
Global Const DIK_E = 18
Global Const DIK_R = 19
Global Const DIK_T = 20
Global Const DIK_Y = 21
Global Const DIK_U = 22
Global Const DIK_I = 23
Global Const DIK_O = 24
Global Const DIK_P = 25
Global Const DIK_LBRACKET = 26
Global Const DIK_RBRACKET = 27
Global Const DIK_RETURN = 28
Global Const DIK_LCONTROL = 29
Global Const DIK_A = 30
Global Const DIK_S = 31
Global Const DIK_D = 32
Global Const DIK_F = 33
Global Const DIK_G = 34
Global Const DIK_H = 35
Global Const DIK_J = 36
Global Const DIK_K = 37
Global Const DIK_L = 38
Global Const DIK_SEMICOLON = 39
Global Const DIK_APOSTROPHE = 40
Global Const DIK_GRAVE = 41
Global Const DIK_LSHIFT = 42
Global Const DIK_BACKSLASH = 43
Global Const DIK_Z = 44
Global Const DIK_X = 45
Global Const DIK_C = 46
Global Const DIK_V = 47
Global Const DIK_B = 48
Global Const DIK_N = 49
Global Const DIK_M = 50
Global Const DIK_COMMA = 51
Global Const DIK_PERIOD = 52
Global Const DIK_SLASH = 53
Global Const DIK_RSHIFT = 54
Global Const DIK_MULTIPLY = 55
Global Const DIK_LALT = 56
Global Const DIK_SPACE = 57
Global Const DIK_CAPSLOCK = 58
Global Const DIK_F1 = 59
Global Const DIK_F2 = 60
Global Const DIK_F3 = 61
Global Const DIK_F4 = 62
Global Const DIK_F5 = 63
Global Const DIK_F6 = 64
Global Const DIK_F7 = 65
Global Const DIK_F8 = 66
Global Const DIK_F9 = 67
Global Const DIK_F10 = 68
Global Const DIK_NUMLOCK = 69
Global Const DIK_SCROLL = 70
Global Const DIK_NUMPAD7 = 71
Global Const DIK_NUMPAD8 = 72
Global Const DIK_NUMPAD9 = 73
Global Const DIK_SUBTRACT = 74
Global Const DIK_NUMPAD4 = 75
Global Const DIK_NUMPAD5 = 76
Global Const DIK_NUMPAD6 = 77
Global Const DIK_ADD = 78
Global Const DIK_NUMPAD1 = 79
Global Const DIK_NUMPAD2 = 80
Global Const DIK_NUMPAD3 = 81
Global Const DIK_NUMPAD0 = 82
Global Const DIK_DECIMAL = 83
Global Const DIK_F11 = 87
Global Const DIK_F12 = 88
Global Const DIK_NUMPADENTER = 156
Global Const DIK_RCONTROL = 157
Global Const DIK_DIVIDE = 181
Global Const DIK_RALT = 184
Global Const DIK_HOME = 199
Global Const DIK_UP = 200
Global Const DIK_PAGEUP = 201
Global Const DIK_LEFT = 203
Global Const DIK_RIGHT = 205
Global Const DIK_END = 207
Global Const DIK_DOWN = 208
Global Const DIK_PAGEDOWN = 209
Global Const DIK_INSERT = 210
Global Const DIK_DELETE = 211
Global Const MOUSE_SPEED = 2

